﻿using System.Reflection;

using NHibernate;
using NHibernate.Cfg;


namespace ThinkNet.Infrastructure
{
    public sealed class NHibernateSessionFactory
    {
        private static object sync = new object();
        private static NHibernateSessionFactory instance;
        /// <summary>
        /// 一个工厂实例
        /// </summary>
        public static NHibernateSessionFactory Instance
        {
            get
            {
                if (instance == null) {
                    lock (sync) {
                        if (instance == null) {
                            instance = new NHibernateSessionFactory();
                        }
                    }
                }

                return instance;
            }
        }


        /// <summary>
        /// The session factory instance.
        /// </summary>
        private ISessionFactory sessionFactory;
        public ISessionFactory SessionFactory
        {
            get
            {
                return sessionFactory;
            }
        }

        /// <summary>
        /// 使用单独的配置文件构造实例
        /// </summary>
        private NHibernateSessionFactory()
            : this(new Configuration())
        { }

        /// <summary>
        /// 根据配置文件构造NHibernateSessionFactory
        /// </summary>
        /// <param name="nhibernateConfig"></param>
        internal NHibernateSessionFactory(Configuration nhibernateConfig)
        {
            sessionFactory = nhibernateConfig.BuildSessionFactory();
        }

        /// <summary>
        /// 获取当前上下文的session
        /// </summary>
        /// <returns></returns>
        public ISession GetSession()
        {
            return sessionFactory.GetCurrentSession();
        }

        /// <summary>
        /// 打开一个新的session
        /// </summary>
        /// <returns></returns>
        public ISession OpenSession()
        {
            return sessionFactory.OpenSession();
        }


        public static void BuildSessionFactory(Configuration nhibernateConfig)
        {
            if (instance == null) {
                try {
                    instance = new NHibernateSessionFactory(nhibernateConfig);
                }
                catch {
                    throw;
                }
            }
        }

        /// <summary>
        /// 使用web.config配置文件创建一个SessionFactory
        /// 因为在web.config里设置mapping属性无效，所以要加入程序集名称
        /// </summary>
        /// <param name="assemblyName"></param>
        public static void BuildSessionFactory(string assemblyName)
        {
            BuildSessionFactory(new Configuration().AddAssembly(assemblyName));
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="assembly"></param>
        public static void BuildSessionFactory(Assembly assembly)
        {
            BuildSessionFactory(new Configuration().AddAssembly(assembly));
        }
    }
}
