﻿using System;
using System.Collections.Generic;
using System.Data.SQLite;
using System.Data.Common;
using System.Data;

namespace ThinkNet.Database
{
    public class SQLiteDatabase : Database
    {
        /// <summary>
        /// 通过有效的数据连接字符串构造实例
        /// </summary>
        /// <param name="connectionString"></param>
        public SQLiteDatabase(string connectionString)
            : base(SQLiteFactory.Instance, connectionString)
        { }

        public SQLiteDatabase(IDbConnection connection) :
            base(connection)
        {
            if (!(connection is SQLiteConnection)) {
                throw new DatabaseException("此数据连接非SQLiteConnection");
            }
        }

        protected override void DeriveParameters(DbCommand discoveryCommand)
        { }

        protected override DbParameter CreateParameter(string paramName, DbType dbType, ParameterDirection direction, int size, object value)
        {
            SQLiteParameter param = new SQLiteParameter();
            param.ParameterName = string.Concat("@", paramName);
            param.DbType = dbType;

            param.Direction = direction;
            if (size > 0)
                param.Size = size;
            if (!(direction == ParameterDirection.Output && value == null))
                param.Value = value;

            return param;
        }

        public override DataSet ExecuteDataset(CommandType commandType, string commandText, params IDataParameter[] commandParameters)
        {
            commandText.MustNotEmpty("commandText");

            using (IDbCommand command = CreateCommandByCommandType(commandType, commandText, commandParameters)) {
                try {
                    using (SQLiteDataAdapter da = new SQLiteDataAdapter()) {
                        //da.MissingSchemaAction = MissingSchemaAction.AddWithKey;
                        da.SelectCommand = command as SQLiteCommand;
                        DataSet ds = new DataSet();

                        // Fill the DataSet using default values for DataTable names, etc
                        da.Fill(ds);

                        // Return the dataset
                        return ds;
                    }
                }
                catch (Exception ex) {
                    throw ex;
                }
                finally {
                    command.Parameters.Clear();
                }
            }
        }

        protected override string GetIdentitySql()
        {
            return "SELECT LAST_INSERT_ROWID()";
        }

        protected override DbColumn[] GetTableColumns(string tableName)
        {
            List<DbColumn> arrField = new List<DbColumn>();

            IDataReader reader = base.ExecuteReader(CommandType.Text,
                string.Format("PRAGMA table_info('{0}')", tableName), (DbParameter[])null);
            while (reader.Read()) {
                DbColumn field = new DbColumn();
                field.ColumnName = reader.GetString(0);
                field.DataType = reader.GetString(1);
                field.AllowDBNull = reader.GetString(2).Equals("yes");
                field.AutoIncrement = reader.GetString(5).Equals("auto_increment");
                field.Primary = reader.GetString(4).Equals("pri");

                arrField.Add(field);
            }
            reader.Close();

            return arrField.ToArray();
        }


        protected override string BuildLikeParameterName(string parameterName)
        {
            return string.Concat("('%'+", parameterName, "+'%')");
        }

        protected override string PagingSupport(int pageIndex, int pageSize)
        {
            return string.Format("SELECT%DISTINCT%%FIELD%%TABLE%%JOIN%%WHERE%%GROUP%%HAVING%%ORDER% LIMIT {0} OFFSET {1}", pageSize, pageIndex * pageSize);
        }
    }
}
