﻿using System;
using System.Linq;
using System.Reflection;

using Microsoft.Practices.Unity;
using Microsoft.Practices.Unity.InterceptionExtension;


namespace ThinkNet.Infrastructure
{
    public class ParticipateInUnityContainerAttribute : ParticipateInContainerAttribute
    {
        public ParticipateInUnityContainerAttribute(params InjectionMember[] injectionMembers)
            : this(new TransientLifetimeManager(), new Type[0], injectionMembers)
        { }

        public ParticipateInUnityContainerAttribute(LifetimeManager lifetimeManager)
            : this(lifetimeManager, new Type[0], new InjectionMember[0])
        { }

        public ParticipateInUnityContainerAttribute(LifetimeManager lifetimeManager, params Type[] registerTypes)
            : this(lifetimeManager, registerTypes, new InjectionMember[0])
        { }

        public ParticipateInUnityContainerAttribute(LifetimeManager lifetimeManager, params InjectionMember[] injectionMembers)
            : this(lifetimeManager, new Type[0], injectionMembers)
        { }

        public ParticipateInUnityContainerAttribute(LifetimeManager lifetimeManager, Type[] registerTypes, params InjectionMember[] injectionMembers)
            : base(registerTypes)
        {
            this.lifetimeManager = lifetimeManager;

            if (injectionMembers != null && injectionMembers.Length > 0) {
                this.injectionMembers = injectionMembers;
            }
        }

        private readonly LifetimeManager lifetimeManager;
        public LifetimeManager LifetimeManager
        {
            get { return this.lifetimeManager; }
        }

        private readonly InjectionMember[] injectionMembers = new InjectionMember[0];
        public InjectionMember[] InjectionMembers
        {
            get { return this.injectionMembers; }
        }

        protected override void RegisterType(IObjectContainer container, Type type, string name)
        {
            if (!injectionMembers.Any()) {
                base.RegisterType(container, type, name);
                return;
            }

            IUnityContainer unityContainer = (container as UnityObjectContainer).UnityContainer;

            var injections = (new InjectionMember[] {
                new Interceptor<VirtualMethodInterceptor>()
            }).Concat(injectionMembers).ToArray();

            if (string.IsNullOrEmpty(name)) {
                unityContainer.RegisterType(type, lifetimeManager, injections);
            }
            else {
                unityContainer.RegisterType(type, name, lifetimeManager, injections);
            }
        }

        protected override void RegisterType(IObjectContainer container, Type from, Type to, string name)
        {
            if (!injectionMembers.Any()) {
                base.RegisterType(container, from, to, name);
                return;
            }

            IUnityContainer unityContainer = (container as UnityObjectContainer).UnityContainer;

            var injections = (new InjectionMember[] {
                new Interceptor<InterfaceInterceptor>(),
                new Interceptor<VirtualMethodInterceptor>()
            }).Concat(injectionMembers).ToArray();

            if (string.IsNullOrEmpty(name)) {
                unityContainer.RegisterType(from, to, lifetimeManager, injections);
            }
            else {
                unityContainer.RegisterType(from, to, name, lifetimeManager, injections);
            }
        }
    }
}
