﻿using System;
using ThinkNet.Domain;
using ThinkNet.Infrastructure;
using ThinkNet.Event;
using ThinkNet.Test.Events;

namespace ThinkNet.Test.Domain
{
    [Serializable]
    public class User : SourcedAggregateRoot<Guid>, IEventHandler<UserCreated>
    {
        public User()
        { }

        public User(string loginId, string password, string userName, string email)
            : base(Guid.NewGuid())
        {
            var userCreated = new UserCreated {
                Email = email,
                UserName = userName,
                Password = password,
                LoginId = loginId
            };

            RaiseEvent(userCreated);
        }

        public string LoginId { get; set; }

        public string Password { get; set; }

        public string UserName { get; set; }

        public string Email { get; set; }


        void IEventHandler<UserCreated>.Handle(UserCreated @event)
        {
            @event.UserID = this.Id;
            this.LoginId = @event.LoginId;
            this.Password = @event.Password;
            this.UserName = @event.UserName;
            this.Email = @event.Email;
        }

        public void LogonToLog(string ip)
        {
            var userLogined = new UserLogined {
                UserId = this.Id,
                ClientIP = ip,
                LoginTime = DateTime.Now
            };

            PublishEvent(userLogined);
        }
    }

    //public class LogonLog : Entity<int>
    //{
    //    public int UserId { get; set; }

    //    public User UserInfo { get; set; }

    //    public string ClientIP { get; set; }

    //    public DateTime LogonTime { get; set; }
    //}
}
