﻿using System.Collections.Generic;
using ThinkNet.Domain;

namespace ThinkNet.Command
{
    /// <summary>
    /// 表示处理命令的一个上下文。
    /// </summary>
    public interface ICommandContext
    {
        /// <summary>
        /// 添加一个新的聚合到当前上下文。
        /// </summary>
        /// <param name="aggregateRoot"></param>
        void Add<TAggregateRoot>(TAggregateRoot aggregateRoot) where TAggregateRoot : class, ISourcedAggregateRoot;
        /// <summary>
        /// 从当前上下文获取一个聚合。
        /// <remarks>
        /// 1. If the aggregate already exist in the current context, then return it directly;
        /// 2. If not exist then try to get it from memory cache;
        /// 3. If still not exist then try to get it from event store;
        /// Finally, if the specified aggregate not found, then AggregateRootNotFoundException will be raised; otherwise, return the found aggregate.
        /// </remarks>
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="id"></param>
        /// <returns></returns>
        TAggregateRoot Get<TAggregateRoot>(object id) where TAggregateRoot : class, ISourcedAggregateRoot;
        ///// <summary>Get an aggregate from the current context.
        ///// <remarks>
        ///// 1. If the aggregate already exist in the current context, then return it directly;
        ///// 2. If not exist then try to get it from memory cache;
        ///// 3. If still not exist then try to get it from event store;
        ///// Finally, if the specified aggregate not found, return null; otherwise, return the found aggregate.
        ///// </remarks>
        ///// </summary>
        ///// <typeparam name="T"></typeparam>
        ///// <param name="id"></param>
        ///// <returns></returns>
        //T GetOrDefault<T>(object id) where T : IAggregateRoot;

        /// <summary>
        /// 获取所有被跟踪的聚合根。
        /// </summary>
        /// <returns></returns>
        IEnumerable<ISourcedAggregateRoot> GetTrackedAggregateRoots();
        /// <summary>
        /// 清除所有被跟踪的聚合根。
        /// </summary>
        void Clear();
    }
}
