﻿using System;
using System.Linq;
using ThinkNet.Infrastructure;

namespace ThinkNet.Command
{
    public class DefaultCommandHandlerProvider : AssemblyInitializer, ICommandHandlerProvider
    {

        public ICommandHandler GetCommandHandler(Type commandType)
        {
            var commandHandlerType = typeof(ICommandHandler<>).MakeGenericType(commandType);
            var commandHandler = ServiceLocator.GetInstance(commandHandlerType);

            if (commandHandler == null)
                return null;

            var commandHandlerWrapperType = typeof(CommandHandlerWrapper<>).MakeGenericType(commandType);

            return (ICommandHandler)Activator.CreateInstance(commandHandlerWrapperType, new[] { commandHandler });
        }


        protected override bool RegisteredTypeName
        {
            get { return false; }
        }

        protected override bool IsGenericType(Type type)
        {
            return IsDefineType(type);
        }

        protected override bool IsRegisterType(Type type)
        {
            return IsCommandHandler(type);
        }

        private static bool IsDefineType(Type type)
        {
            return type.IsGenericType && type.GetGenericTypeDefinition() == typeof(ICommandHandler<>);
        }

        public static bool IsCommandHandler(Type type)
        {
            return type.IsClass && !type.IsAbstract &&
                type.GetInterfaces().Any(IsDefineType);
        }
    }
}
