﻿using System.Configuration;

namespace ThinkNet.Database
{
    /// <summary>
    /// Database配置节点
    /// </summary>
    public sealed class DatabaseConfigSection : ConfigurationSection
    {
        /// <summary>
        /// 获取配置信息
        /// </summary>
        /// <returns></returns>
        public static DatabaseConfigSection GetSection(string prefix = null)
        {
            if (string.IsNullOrEmpty(prefix)) {
                prefix = SectionPrefix;
            }


            object obj = ConfigurationManager.GetSection(prefix);
            if (obj != null && obj is DatabaseConfigSection) {
                return (DatabaseConfigSection)obj;
            }

            throw new DatabaseException("配置错误。");
        }

        /// <summary>
        /// 默认的配置节点名称
        /// </summary>
        public const string SectionPrefix = "databaseConfig";


        private static readonly ConfigurationProperty _propProvider = new ConfigurationProperty("provider", typeof(string), null, ConfigurationPropertyOptions.IsRequired);
        /// <summary>
        /// 提供都
        /// </summary>
        [ConfigurationProperty("provider", Options = ConfigurationPropertyOptions.IsRequired)]
        public string ProviderName
        {
            get { return (string)base[_propProvider]; }
            set { base[_propProvider] = value; }
        }

        private static readonly ConfigurationProperty _propConnectionString = new ConfigurationProperty("connectionString", typeof(string), null);
        /// <summary>
        /// 数据库连接字符串
        /// </summary>
        [ConfigurationProperty("connectionString")]
        public string ConnectionString
        {
            get { return (string)base[_propConnectionString]; }
            set { base[_propConnectionString] = value; }
        }

        private static readonly ConfigurationProperty _propConnectionStringName = new ConfigurationProperty("connectionStringName", typeof(string), null);
        /// <summary>
        /// 数据库连接字符串名称
        /// </summary>
        [ConfigurationProperty("connectionStringName")]
        public string ConnectionStringName
        {
            get { return (string)base[_propConnectionStringName]; }
            set { base[_propConnectionStringName] = value; }
        }
    }
}
