﻿using System;
using System.Configuration;
using System.Data;

namespace ThinkNet.Database
{
    /// <summary>
    /// Database工厂
    /// </summary>
    public static class DatabaseFactory
    {
        /// <summary>
        /// 根据数据连接字符串创建一个Database
        /// </summary>
        /// <param name="connectionString"></param>
        /// <returns></returns>
        public static IDatabase CreateDatabase(string connectionString)
        {
            var config = DatabaseConfigSection.GetSection();
            var type = BuildDatabaseType(config);

            return (IDatabase)Activator.CreateInstance(type, connectionString);
        }

        /// <summary>
        /// 根据Database配置节点创建一个Database
        /// </summary>
        /// <returns></returns>
        public static IDatabase CreateDatabase()
        {
            var config = DatabaseConfigSection.GetSection();
            var type = BuildDatabaseType(config);

            if (string.IsNullOrEmpty(config.ConnectionString) && string.IsNullOrEmpty(config.ConnectionStringName))
                throw new DatabaseException("至少配置 connectionString 或 connectionStringName 其中一项。");

            string connectionString;
            if(!string.IsNullOrEmpty(config.ConnectionString))
                connectionString = config.ConnectionString;
            else 
                connectionString = ConfigurationManager.ConnectionStrings[config.ConnectionStringName].ConnectionString;

            return (IDatabase)Activator.CreateInstance(type, connectionString);
        }

        /// <summary>
        /// 根据一个有效的数据连接创建一个Database
        /// </summary>
        /// <param name="connection"></param>
        /// <returns></returns>
        public static IDatabase CreateDatabase(IDbConnection connection)
        {
            var config = DatabaseConfigSection.GetSection();
            var type = BuildDatabaseType(config);

            return (IDatabase)Activator.CreateInstance(type, connection);
        }

        private static Type BuildDatabaseType(DatabaseConfigSection config)
        {
            Type type;
            try {
                type = Type.GetType(config.ProviderName, false, true);
            }
            catch (Exception ex) {
                throw ex;
            }

            if (!typeof(IDatabase).IsAssignableFrom(type)) {
                throw new DatabaseException("该对象没有继承 IDatabase 接口。");
            }

            return type;
        }
    }
}
