﻿using System;
using System.Collections;
#if NET40
using System.Collections.Concurrent;
#endif
using System.Collections.Generic;
using System.Linq;
using System.Reflection;

namespace ThinkNet.Database
{
    internal static class EntityUtils
    {
#if NET40
        static ConcurrentDictionary<Type, PropertyInfo[]> _entityPropertiesCache = new ConcurrentDictionary<Type, PropertyInfo[]>();
        public static PropertyInfo[] GetProperties(Type type)
        {
            PropertyInfo[] cachedProperties;

            if (!_entityPropertiesCache.TryGetValue(type, out cachedProperties)) {
                cachedProperties = type.GetProperties(BindingFlags.Public | BindingFlags.Instance);

                _entityPropertiesCache.TryAdd(type, cachedProperties);
            }

            return cachedProperties;
        }

        static ConcurrentDictionary<Type, IDictionary<string, string>> _entityMappingsCache = new ConcurrentDictionary<Type, IDictionary<string, string>>();
        public static IDictionary<string, string> GetMappings(Type type)
        {
            IDictionary<string, string> cachedMappings;

            if (!_entityMappingsCache.TryGetValue(type, out cachedMappings)) {
                cachedMappings = new Dictionary<string, string>();

                foreach (PropertyInfo property in GetProperties(type)) {
                    if (property.IsDefined(typeof(FieldMappingAttribute), false)) {
                        cachedMappings.Add(property.Name, property.GetAttribute<FieldMappingAttribute>(false).FieldName);
                    }
                    else {
                        cachedMappings.Add(property.Name, property.Name);
                    }
                }

                _entityMappingsCache.TryAdd(type, cachedMappings);
            }


            return cachedMappings;
        }
#endif

#if NET35
        static Hashtable _entityInfosCache = Hashtable.Synchronized(new Hashtable());

        public static PropertyInfo[] GetProperties(Type type)
        {
            string hashKey = string.Format("EP:{0}", type.FullName);

            PropertyInfo[] cachedProperties = _entityInfosCache[hashKey] as PropertyInfo[];
            if (cachedProperties == null) {
                cachedProperties = type.GetProperties(BindingFlags.Public | BindingFlags.Instance);

                _entityInfosCache[hashKey] = cachedProperties;
            }

            return cachedProperties;
        }

        public static Dictionary<string, string> GetMappings(Type type)
        {
            string hashKey = string.Format("EM:{0}", type.FullName);

            Dictionary<string, string> cachedMappings = _entityInfosCache[hashKey] as Dictionary<string, string>;

            if (cachedMappings == null) {
                cachedMappings = new Dictionary<string, string>();
                //PropertyInfo[] properties = GetProperties(type);//.Where(p => !p.IsDefined(typeof(FieldIgnoreAttribute), false));
                foreach (PropertyInfo property in GetProperties(type)) {
                    if (property.IsDefined(typeof(FieldMappingAttribute), false)) {
                        cachedMappings.Add(property.Name, property.GetAttribute<FieldMappingAttribute>(false).FieldName);
                    }
                    else {
                        cachedMappings.Add(property.Name, property.Name);
                    }
                }

                _entityInfosCache[hashKey] = cachedMappings;
            }

            return cachedMappings;
        }
#endif
    }
}