﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Text;

namespace ThinkNet.Database
{
    /// <summary>
    /// 实现各种查询条件的类
    /// </summary>
    internal static class Restrictions
    {
        #region
        internal static Dictionary<DbJudgeSymbol, string> symbol = new Dictionary<DbJudgeSymbol, string>();
        static Restrictions()
        {
            if (symbol.Count == 0) {
                symbol.Add(DbJudgeSymbol.GT, " > ");
                symbol.Add(DbJudgeSymbol.EGT, " >= ");
                symbol.Add(DbJudgeSymbol.LT, " < ");
                symbol.Add(DbJudgeSymbol.ELT, " <= ");
                symbol.Add(DbJudgeSymbol.IN, " IN ");
                symbol.Add(DbJudgeSymbol.NIN, " NOT IN ");
                symbol.Add(DbJudgeSymbol.LIKE, " LIKE ");
                //symbol.Add(DbJudgeSymbol.NLIKE, " NOT LIKE ");
                symbol.Add(DbJudgeSymbol.EQ, " = ");
                symbol.Add(DbJudgeSymbol.NEQ, " <> ");
                //symbol.Add(DbJudgeSymbol.BETWEEN, " BETWEEN ");
                //symbol.Add(DbJudgeSymbol.NBETWEEN, " NOT BETWEEN ");
                //symbol.Add(0, string.Empty);
            }
        }
        #endregion

        #region
        private static string OutputNormal(object data)
        {
            if (data == null)
                return "NULL";


            Type type = data.GetType();
            if (type == typeof(bool)) {
                return Convert.ToInt32(data).ToString();
            }
            if (type == typeof(int) || type == typeof(decimal) || type == typeof(byte) || type == typeof(short) || type == typeof(long) || type == typeof(float) || type == typeof(double)) {
                return data.ToString();
            }

            string str = Convert.ToString(data, CultureInfo.CurrentCulture).Trim();

            return string.Format("'{0}'", str.Trim().Replace("'", ""));
        }
        #endregion

        internal static string OutputSqlString(SqlOrder[] orders)
        {
            int total = orders.Length;
            if (total == 0)
                return string.Empty;

            StringBuilder sb = new StringBuilder(" ORDER BY ");

            for (int i = 0; i < total; i++) {
                if (i > 0 && i < total) {
                    sb.Append(",");
                }

                sb.AppendFormat("{0} {1}", orders[i].FieldName, orders[i].OrderType);
            }


            return sb.ToString();
        }

        internal static string OutputSqlString(SqlGroup[] groups)
        {
            int total = groups.Length;
            if (total == 0)
                return string.Empty;

            StringBuilder sb = new StringBuilder(" GROUP BY ");

            for (int i = 0; i < total; i++) {
                if (i > 0 && i < total) {
                    sb.Append(",");
                }

                sb.Append(groups[i].FieldName);
            }


            return sb.ToString();
        }

        internal static string OutputSqlString(SqlField[] fields)
        {
            int total = fields.Length;
            if (total == 0)
                return " * ";

            StringBuilder sb = new StringBuilder(" ");
            for (int i = 0; i < total; i++) {
                if (i > 0 && i < total) {
                    sb.Append(",");
                }

                SqlField col = fields[i];

                if (col.Aggrefunc != DbAggrefunc.NONE) {
                    sb.Append("(").Append(col.Aggrefunc).Append(")");
                }
                else {
                    sb.Append(col.FieldName);
                }

                if (!string.IsNullOrEmpty(col.Alias)) {
                    sb.AppendFormat(" AS {0}", col.Alias);
                }
            }

            return sb.ToString();
        }
    }
}
