﻿using System.Collections.Generic;
using System.Linq;

namespace ThinkNet.Database
{
    public class SqlJoin
    {
        /// <summary>
        /// 创建新实例
        /// </summary>
        /// <param name="tableName">连接表名称</param>
        public SqlJoin(string tableName) :
            this(DbJoinWay.INNER, tableName)
        { }


        /// <summary>
        /// 创建新实例
        /// </summary>
        /// <param name="joinType">连接方式</param>
        /// <param name="tableName">连接表名称</param>
        public SqlJoin(DbJoinWay joinType, string tableName)
        {
            this.JoinType = joinType;
            this.Table = tableName;
        }

        /// <summary>
        /// 设置连接查询(主外键)
        /// </summary>
        /// <param name="sameField">相同字段名</param>
        /// <param name="joinTable">连接表名</param>
        public void On(string sameField, string joinTable)
        {
            this.On(sameField, joinTable, sameField);
        }

        /// <summary>
        /// 设置连接查询(主外键)
        /// </summary>
        /// <param name="field">当前字段名</param>
        /// <param name="joinTable">连接表名</param>
        /// <param name="joinField">连接表字段名</param>
        public void On(string field, string joinTable, string joinField)
        {
            SqlWhere condition = new SqlWhere();
            condition.FieldName = string.Format("{0}.{1}", Table, field);
            condition.Symbol = DbJudgeSymbol.EQ;
            condition.Safety = true;
            condition.Logical =  DbLogical.And;
            condition.Value = string.Format("{0}.{1}", joinTable, joinField);
            //query.Current(DbJudgeSymbol.EQ, string.Format("{0}.{1}", joinTable, joinField));

            this.On(condition);
        }

        SqlWhere[] conditions = new SqlWhere[0];
        /// <summary>
        /// 连接查询条件
        /// </summary>
        /// <param name="conditions">查询条件</param>
        public void On(params SqlWhere[] conditions)
        {
            if (conditions != null && conditions.Length > 0) {
                this.conditions = conditions;
            }
        }

        public void On(IEnumerable<SqlWhere> conditions)
        {
            this.On(conditions.ToArray());
        }

        /// <summary>
        /// 连接方式
        /// </summary>
        public DbJoinWay JoinType { get; internal set; }

        /// <summary>
        /// 表名
        /// </summary>
        public string Table { get; internal set; }


        /// <summary>
        /// 查询条件
        /// </summary>
        public SqlWhere[] OnConditions { get; internal set; }
    }
}
