﻿using System;
using System.Runtime.ConstrainedExecution;

namespace ThinkNet
{
    /// <summary>
    /// 表示派生对象是一次性的
    /// </summary>
    public abstract class DisposableObject : CriticalFinalizerObject, IDisposable
    {
        /// <summary>
        /// 析构函数
        /// </summary>
        ~DisposableObject()
        {
            this.Dispose(false);
        }

        /// <summary>
        /// 释放资源(自定义)
        /// </summary>
        /// <param name="disposing">表示对象是否应该明确处理</param>
        protected abstract void Dispose(bool disposing);

        /// <summary>
        /// 释放资源
        /// </summary>
        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }
    }
}
