﻿using System.Collections.Generic;

using ThinkNet.Event;
using ThinkNet.Infrastructure;


namespace ThinkNet.Domain
{
    /// <summary>
    /// 表示继承该接口的类是一个聚合根
    /// </summary>
    public interface IAggregateRoot : IEntity
    {
        ///// <summary>
        ///// 版本
        ///// </summary>
        //int Version { get; }

        /// <summary>
        /// 获取所有未经提交的领域事件。
        /// </summary>
        IEnumerable<IEvent> UncommittedEvents { get; }

        /// <summary>
        /// 当完成领域事件向外部系统提交后，清除所有已产生的领域事件。
        /// </summary>
        void ClearEvents();

        ///// <summary>
        ///// 通过事件还原聚合根
        ///// </summary>
        ///// <param name="eventStreams"></param>
        //void ReplayEventStreams(IEnumerable<EventStream> eventStreams);
    }

    /// <summary>
    /// 该接口的类是一个可用事件追溯还原的聚合根
    /// </summary>
    public interface ISourcedAggregateRoot : IAggregateRoot
    {
        /// <summary>
        /// 版本号
        /// </summary>
        int Version { get; }

        /// <summary>
        /// 通过事件还原聚合根
        /// </summary>
        /// <param name="eventStreams"></param>
        void ReplayEventStreams(IEnumerable<EventStream> eventStreams);
    }

    ///// <summary>
    ///// 表示继承该接口的类是一个聚合根
    ///// </summary>
    ///// <typeparam name="TIdentify">标识类型</typeparam>
    //public interface IAggregateRoot<TIdentify> : IEntity<TIdentify>, IAggregateRoot
    //{ }
}
