﻿using System;

namespace ThinkNet.Domain
{
    /// <summary>
    /// 仓储接口
    /// </summary>
    public interface IRepository
    {
        /// <summary>
        /// 根据聚合根类型和标识获取一个聚合
        /// </summary>
        /// <param name="type"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        ISourcedAggregateRoot Get(Type type, object id);
        /// <summary>Get an aggregate from memory cache, if not exist, get it from event store.
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        TAggregateRoot Get<TAggregateRoot>(object id) where TAggregateRoot : class, ISourcedAggregateRoot;

        /// <summary>
        /// 保存聚合根。需要刷新Cache。也可以保存一份快照
        /// </summary>
        /// <param name="aggregateRoot"></param>
        void SaveOrUpdate(ISourcedAggregateRoot aggregateRoot);

        /// <summary>
        /// 删除聚合根。从Cache中移除。也可以删除该聚合相关的事件和快照。
        /// </summary>
        /// <param name="aggregateRoot"></param>
        void Delete(ISourcedAggregateRoot aggregateRoot);
    }
}
