﻿using System;
using System.Collections;

using ThinkNet.Database;
using ThinkNet.Infrastructure;


namespace ThinkNet.Domain
{
    public class DatabaseSnapshotStore : ISnapshotStore
    {
        private readonly IDatabase _database;
        private readonly IStringSerializer _serializer;
        private readonly ISnapshotTableNameProvider _snapshotTableProvider;
        private readonly IAggregateRootTypeProvider _aggregateRootTypeProvider;

        public DatabaseSnapshotStore(IDatabase database, IStringSerializer serializer,
            ISnapshotTableNameProvider snapshotTableProvider, IAggregateRootTypeProvider aggregateRootTypeProvider)
        {
            this._database = database;
            this._serializer = serializer;
            this._snapshotTableProvider = snapshotTableProvider;
            this._aggregateRootTypeProvider = aggregateRootTypeProvider;
        }

        

        //public bool IsEventStreamExist(string aggregateRootId, Type aggregateRootType, Guid id)
        //{
        //    var tableName = _eventTableProvider.GetTable(aggregateRootType);

        //    return _database.Table(tableName).Where("Id", id).Count() > 0;
        //}

        #region ISnapshotStore 成员

        public void SaveShapshot(Snapshot snapshot)
        {
            var aggregateRootType = _aggregateRootTypeProvider.GetAggregateRootType(snapshot.AggregateRootName);
            var snapshotTable = _snapshotTableProvider.GetTable(aggregateRootType);
            var data = BuildDictionaryFromSnapshot(snapshot);

            _database.Table(snapshotTable).Data(data).Save();
        }

        public void Remove(object aggregateRootId, Type aggregateRootType)
        {
            var snapshotTable = _snapshotTableProvider.GetTable(aggregateRootType);

            SqlWhere[] conditions = {
                                        new SqlWhere("AggregateRootId", aggregateRootId), 
                                        new SqlWhere("AggregateRootName", aggregateRootType.FullName)
                                    };

            _database.Table(snapshotTable).Where(conditions).Delete();
        }

        public Snapshot GetLastestSnapshot(object aggregateRootId, Type aggregateRootType)
        {
            var snapshotTable = _snapshotTableProvider.GetTable(aggregateRootType);

            SqlWhere[] conditions = {
                                        new SqlWhere("AggregateRootId", aggregateRootId), 
                                        new SqlWhere("AggregateRootName", aggregateRootType.FullName),
                                    };

            var result = _database.Table(snapshotTable).Where(conditions).OrderBy("Version", DbOrderType.DESC).Find();

            return BuildSnapshotFromDictionary(result);
        }

        #endregion

        private Snapshot BuildSnapshotFromDictionary(IDictionary dict)
        {
            if (dict == null || dict.Count == 0) return null;

            var snapshot = new Snapshot();
            snapshot.AggregateRootId = (string)dict["AggregateRootId"];
            snapshot.AggregateRootName = (string)dict["AggregateRootName"];
            snapshot.Version = (int)dict["Version"];
            snapshot.Timestamp = (DateTime)dict["Timestamp"];
            snapshot.Data = _serializer.Deserialize<ISourcedAggregateRoot>((string)dict["Data"]);


            return snapshot;
        }

        private IDictionary BuildDictionaryFromSnapshot(Snapshot snapshot)
        {
            snapshot.MustNotNull("snapshot");

            Hashtable ht = new Hashtable();
            ht["Id"] = Guid.NewGuid();
            ht["AggregateRootId"] = snapshot.AggregateRootId;
            ht["AggregateRootName"] = snapshot.AggregateRootName;
            ht["Version"] = snapshot.Version;
            ht["Timestamp"] = DateTime.UtcNow;
            ht["Data"] = _serializer.Serialize(snapshot.Data);

            return ht;
        }
    }
}
