﻿using System;
using System.Collections;
using System.Collections.Generic;
#if NET40
using System.Collections.Concurrent;
#endif

using ThinkNet.Infrastructure;


namespace ThinkNet.Domain
{
    public class MemoryCache : ICache
    {
#if NET40
        private readonly ConcurrentDictionary<string, byte[]> _cacheDict = new ConcurrentDictionary<string, byte[]>();
#endif
#if NET35
        private readonly Dictionary<string, byte[]> _cacheDict = new Dictionary<string, byte[]>();
#endif

        private IBinarySerializer _binarySerializer;

        public MemoryCache(IBinarySerializer binarySerializer)
        {
            _binarySerializer = binarySerializer;
        }

        public ISourcedAggregateRoot Get(object id, Type type)
        {
            id.MustNotNull("id");

            string key = GetUniqueKey(type, id);
            byte[] value;
            return _cacheDict.TryGetValue(key, out value) ? (ISourcedAggregateRoot)_binarySerializer.Deserialize(value) : null;
        }

        public TAggregateRoot Get<TAggregateRoot>(object id) where TAggregateRoot : class, ISourcedAggregateRoot
        {
            id.MustNotNull("id");

            string key = GetUniqueKey(typeof(TAggregateRoot), id);
            byte[] value;
            return _cacheDict.TryGetValue(key, out value) ? _binarySerializer.Deserialize<TAggregateRoot>(value) : null;
        }

        public void Set(ISourcedAggregateRoot aggregateRoot)
        {
            aggregateRoot.MustNotNull("aggregateRoot");

            string key = GetUniqueKey(aggregateRoot.GetType(), aggregateRoot.UniqueId);
#if NET35
            lock (((ICollection)_cacheDict).SyncRoot)
                _cacheDict[key] = _binarySerializer.Serialize(aggregateRoot);
#endif

#if NET40
            _cacheDict[key] = _binarySerializer.Serialize(aggregateRoot);
#endif
        }

        public void Remove(object id, Type type)
        {
            id.MustNotNull("id");

            byte[] value;
            string key = GetUniqueKey(type, id);

#if NET35
            lock (((ICollection)_cacheDict).SyncRoot) {
                if (_cacheDict.TryGetValue(key, out value)) {
                    _cacheDict.Remove(key);
                }
            }

            return value == null ? null : (IAggregateRoot)_binarySerializer.Deserialize(value);
#endif

#if NET40            
            _cacheDict.TryRemove(key, out value);
#endif
        }

        private static string GetUniqueKey(Type aggregateRootType, object id)
        {
            return string.Concat(aggregateRootType.FullName.ToLower(), ":", id.ToString());
        }
    }
}
