﻿using System;
using System.Collections.Generic;
using ThinkNet.Messaging;

namespace ThinkNet.Event
{
    /// <summary>
    /// 事件流信息
    /// </summary>
    [Serializable]
    public class EventStream : Message
    {
        public EventStream()
        { }

        /// <summary>Parameterized constructor.
        /// </summary>
        /// <param name="aggregateRootId"></param>
        /// <param name="aggregateRootName"></param>
        /// <param name="version"></param>
        /// <param name="commandId"></param>
        /// <param name="timestamp"></param>
        /// <param name="events"></param>
        public EventStream(object aggregateRootId, string aggregateRootName, int version, DateTime timestamp, IEnumerable<IEvent> events)
            : this(Guid.NewGuid(), aggregateRootId, aggregateRootName, version, timestamp, events)
        { }
        /// <summary>Parameterized constructor.
        /// </summary>
        /// <param name="id"></param>
        /// <param name="aggregateRootId"></param>
        /// <param name="aggregateRootName"></param>
        /// <param name="version"></param>
        /// <param name="commandId"></param>
        /// <param name="timestamp"></param>
        /// <param name="events"></param>
        public EventStream(Guid id, object aggregateRootId, string aggregateRootName, int version, DateTime timestamp, IEnumerable<IEvent> events)
            : base(id, timestamp)
        {
            this.AggregateRootId = aggregateRootId;
            this.AggregateRootName = aggregateRootName;
            this.Version = version;
            this.Events = events ?? new List<IEvent>();
        }


        /// <summary>
        /// 聚合根标识。
        /// </summary>
        public object AggregateRootId { get; set; }
        /// <summary>
        /// 聚合根类型名称。
        /// </summary>
        public string AggregateRootName { get; set; }
        ///// <summary>The command id.
        ///// </summary>
        //public Guid CommandId { get; private set; }
        /// <summary>
        /// 事件流版本号。
        /// </summary>
        public int Version { get; set; }
        ///// <summary>The occurred time of the event stream.
        ///// </summary>
        //public DateTime Timestamp { get; set; }
        /// <summary>
        /// 聚合根事件集合。
        /// </summary>
        public IEnumerable<IEvent> Events { get; set; }
    }
}
