﻿#if NET40
using System.Threading.Tasks;
#endif

namespace ThinkNet.Event
{
    /// <summary>Represents a event handler.
    /// </summary>
    public interface IEventHandler
    {
        /// <summary>Handle the given event.
        /// </summary>
        /// <param name="event"></param>
        void Handle(IEvent @event);

#if NET40
        Task HandleAsync(IEvent @event);
#endif

        /// <summary>Get the inner event handler.
        /// </summary>
        /// <returns></returns>
        object GetInnerEventHandler();
    }



    /// <summary>
    /// 表示实现该接口的类型为事件处理器。
    /// </summary>
    /// <typeparam name="TEvent">事件的类型。</typeparam>
    public interface IEventHandler<TEvent> where TEvent : class, IEvent
    {
        /// <summary>
        /// 处理给定的事件。
        /// </summary>
        /// <param name="event">需要处理的事件。</param>
        void Handle(TEvent @event);
    }
}
