﻿
namespace ThinkNet.Event
{
    /// <summary>Represents a event persistence synchronizer.
    /// <remarks>
    ///  Code can be executed before and after the event persistence.
    /// </remarks>
    /// </summary>
    public interface IEventSynchronizer
    {
        /// <summary>Executed before persisting the event.
        /// </summary>
        void OnBeforePersisting(IEvent @event);
        /// <summary>Executed after the event was persisted.
        /// </summary>
        void OnAfterPersisted(IEvent @event);
        /// <summary>Represents the inner generic IEventSynchronizer.
        /// </summary>
        object GetInnerSynchronizer();
    }

    /// <summary>
    /// 事件持久化同步器。
    /// <remarks>
    /// Code can be executed before and after the event persistence.
    /// </remarks>
    /// </summary>
    /// <typeparam name="TEvent"></typeparam>
    public interface IEventSynchronizer<TEvent> where TEvent : class, IEvent
    {
        /// <summary>
        /// 事件持久化之前执行。
        /// </summary>
        void OnBeforePersisting(TEvent @event);
        /// <summary>
        /// 事件持久化之后执行。
        /// </summary>
        void OnAfterPersisted(TEvent @event);
    }
}
