﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;

using ThinkNet.Infrastructure;
using ThinkNet.Domain;

namespace ThinkNet.Event
{
    public class DefaultEventHandlerProvider : AssemblyInitializer, IEventHandlerProvider
    {        
        #region IEventHandlerProvider 成员

        public IEnumerable<IEventHandler> GetEventHandlers(Type eventType)
        {
            var eventHandlerType = typeof(IEventHandler<>).MakeGenericType(eventType);

            return ServiceLocator.GetAllInstances(eventHandlerType)
                .Select(eventHandler => {
                    var eventHandlerWrapperType = typeof(EventHandlerWrapper<>).MakeGenericType(eventType);

                    return Activator.CreateInstance(eventHandlerWrapperType, new[] { eventHandler });
                }).Cast<IEventHandler>();
        }

        #endregion

        protected override bool IsGenericType(Type type)
        {
            return IsDefineType(type);
        }

        protected override bool IsRegisterType(Type type)
        {
            return IsEventHandler(type);
        }

        private static bool IsDefineType(Type type)
        {
            return type.IsGenericType && type.GetGenericTypeDefinition() == typeof(IEventHandler<>);
        }

        public static bool IsEventHandler(Type type)
        {
            return type.IsClass && !type.IsAbstract &&
                !typeof(IAggregateRoot).IsAssignableFrom(type) &&
                type.GetInterfaces().Any(IsDefineType);
        }
    }
}
