﻿using System;
using System.Collections.Generic;
using System.Linq;
using ThinkNet.Infrastructure;

namespace ThinkNet.Event
{
    public class DefaultEventSynchronizerProvider : AssemblyInitializer, IEventSynchronizerProvider
    {
        public IEnumerable<IEventSynchronizer> GetSynchronizers(Type eventType)
        {
            var eventHandlerType = typeof(IEventSynchronizer<>).MakeGenericType(eventType);

            return ServiceLocator.GetAllInstances(eventHandlerType)
                .Select(eventHandler => {
                    var eventHandlerWrapperType = typeof(EventSynchronizerWrapper<>).MakeGenericType(eventType);

                    return Activator.CreateInstance(eventHandlerWrapperType, new[] { eventHandler });
                }).Cast<IEventSynchronizer>();
        }

        protected override bool IsGenericType(Type type)
        {
            return IsDefineType(type);
        }

        protected override bool IsRegisterType(Type type)
        {
            return IsEventSynchronizer(type);
        }

        private static bool IsDefineType(Type type)
        {
            return type.IsGenericType && type.GetGenericTypeDefinition() == typeof(IEventSynchronizer<>);
        }

        public static bool IsEventSynchronizer(Type type)
        {
            return type.IsClass && !type.IsAbstract &&
                type.GetInterfaces().Any(IsDefineType);
        }
    }
}
