﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;

using ThinkNet.Domain;


namespace ThinkNet.Event
{
    public class DefaultEventTableNameProvider : IEventTableNameProvider
    {
        private readonly IAggregateRootTypeProvider _aggregateRootTypeProvider;

        /// <summary>Parameterized constructor.
        /// </summary>
        /// <param name="aggregateRootTypeProvider"></param>
        public DefaultEventTableNameProvider(IAggregateRootTypeProvider aggregateRootTypeProvider)
        {
            _aggregateRootTypeProvider = aggregateRootTypeProvider;
        }


        #region IEventTableNameProvider 成员

        public string GetTable(Type aggregateRootType)
        {
            return GetDefineTableName(aggregateRootType);
        }

        public IEnumerable<string> GetAllTables()
        {
            return _aggregateRootTypeProvider.GetAllAggregateRootTypes().Select<Type, string>(GetDefineTableName).Distinct().ToArray();
        }

        #endregion

        private static string GetDefineTableName(Type type)
        {
            if (type.IsDefined(typeof(EventTableNameAttribute), false)) {
                var attribute = type.GetAttribute<EventTableNameAttribute>(false);

                return attribute.PerAggregate ? string.Concat("Event_", type.Name) : attribute.TableName;
            }
            else {
                return "Events";
            }
        }
    }
}
