﻿using System;
using System.Collections.Generic;
using ThinkNet.Database;

namespace ThinkNet.Event
{
    public class EventHandleInfoInDatabase : EventHandleInfoInMemory
    {
        private readonly string _connectionString;
        private readonly string _tableName;


        public EventHandleInfoInDatabase(string connectionString, string tableName)
        {
            connectionString.MustNotEmpty("connectionString");
            tableName.MustNotEmpty("tableName");

            _connectionString = connectionString;
            _tableName = tableName;
        }

        public override void AddEventHandleInfo(Guid eventId, string eventHandlerTypeName)
        {
            base.AddEventHandleInfo(eventId, eventHandlerTypeName);

            using (var db = DatabaseFactory.CreateDatabase(_connectionString)) {
                if (!EventHandleInfoExist(db, _tableName, new { EventHandlerTypeName = eventHandlerTypeName, EventId = eventId })) {
                    var dict = new Dictionary<string, object>();
                    dict.Add("EventId", eventId);
                    dict.Add("EventHandlerTypeName", eventHandlerTypeName);

                    db.Table(_tableName).Data(dict).Save();
                }
            }
        }

        public override bool IsEventHandleInfoExist(Guid eventId, string eventHandlerTypeName)
        {
            var exist = base.IsEventHandleInfoExist(eventId, eventHandlerTypeName);

            if (!exist) {
                using (var db = DatabaseFactory.CreateDatabase(_connectionString)) {
                    return EventHandleInfoExist(db, _tableName, new { EventHandlerTypeName = eventHandlerTypeName, EventId = eventId });
                }
            }

            return exist;
        }

        private static bool EventHandleInfoExist(IDatabase database, string table, dynamic content)
        {
            SqlWhere[] conditions = {
                                        new SqlWhere("EventId", (Guid)content.EventId),
                                        new SqlWhere("EventHandlerTypeName", (string)content.EventHandlerTypeName)
                                    };

            return database.Table(table).Where(conditions).Count() > 0;
        }
    }
}
