﻿using System;
using System.Collections.Generic;
using System.Collections.Concurrent;

namespace ThinkNet.Event
{
    public class EventHandleInfoInMemory : IEventHandleInfoStore
    {
        private readonly ConcurrentDictionary<Guid, IList<string>> _handleInfoDict = new ConcurrentDictionary<Guid, IList<string>>();

        /// <summary>Insert an event handle info to the cache.
        /// </summary>
        /// <param name="eventId"></param>
        /// <param name="eventHandlerTypeName"></param>
        public virtual void AddEventHandleInfo(Guid eventId, string eventHandlerTypeName)
        {
            IList<string> handlerTypeNameList;
            if (!_handleInfoDict.TryGetValue(eventId, out handlerTypeNameList)) {
                handlerTypeNameList = new List<string>();
            }
            handlerTypeNameList.Add(eventHandlerTypeName);
        }
        /// <summary>Check whether the given event was handled by the given event handler.
        /// </summary>
        /// <param name="eventId"></param>
        /// <param name="eventHandlerTypeName"></param>
        /// <returns></returns>
        public virtual bool IsEventHandleInfoExist(Guid eventId, string eventHandlerTypeName)
        {
            IList<string> handlerTypeNameList;
            return _handleInfoDict.TryGetValue(eventId, out handlerTypeNameList) && handlerTypeNameList.Contains(eventHandlerTypeName);
        }
        /// <summary>Remove all the event handle information from the cache by the given eventId.
        /// </summary>
        /// <param name="eventId"></param>
        public virtual void RemoveEventHandleInfo(Guid eventId)
        {
            IList<string> handlerTypeNameList;
            _handleInfoDict.TryRemove(eventId, out handlerTypeNameList);
        }
    }
}
