﻿using System;
#if NET40
using System.Threading.Tasks;
#endif

using ThinkNet.Infrastructure;

namespace ThinkNet.Event
{
    /// <summary>The default implementation of IEventHandler.
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class EventHandlerWrapper<T> : IEventHandler
        where T : class, IEvent
    {
        private readonly IEventHandler<T> _eventHandler;

        /// <summary>Parameterized constructor.
        /// </summary>
        /// <param name="eventHandler"></param>
        public EventHandlerWrapper(IEventHandler<T> eventHandler)
        {
            _eventHandler = eventHandler;
        }

        /// <summary>Handle the given event.
        /// </summary>
        /// <param name="@event"></param>
        public void Handle(IEvent @event)
        {
            _eventHandler.Handle(@event as T);
        }

#if NET40
        public Task HandleAsync(IEvent @event)
        {
            return Task.Factory.StartNew(() => {
                try {
                    this.Handle(@event);
                }
                catch (Exception ex) {
                    ServiceLocator.GetInstance<ILogger>().Error(ex);
                }
            });
        }
#endif

        /// <summary>Get the inner event handler.
        /// </summary>
        /// <returns></returns>
        public object GetInnerEventHandler()
        {
            return _eventHandler;
        }
    }
}
