﻿using System.Linq;

namespace System.Reflection
{
    public static class CustomAttributeProviderExtentions
    {
        public static TAttribute[] GetAttributes<TAttribute>(this ICustomAttributeProvider provider, bool inherit)
            where TAttribute : Attribute
        {
            return provider
                .GetCustomAttributes(typeof(TAttribute), inherit)
                .Cast<TAttribute>()
                .ToArray();
        }


        /// <summary>
        /// 返回<paramref name="provider"/>上定义的第一个<typeparamref name="TAttribute"/>属性。
        /// </summary>
        public static TAttribute GetAttribute<TAttribute>(this ICustomAttributeProvider provider, bool inherit)
            where TAttribute : Attribute
        {
            return provider.IsDefined(typeof(TAttribute), inherit)
                    ? provider.GetAttributes<TAttribute>(inherit)[0]
                    : default(TAttribute);
        }
    }
}
