﻿
namespace System.Threading
{
    /// <summary>An extension class to provide utility lock mechanism.
    /// </summary>
    public static class ReaderWriterLockSlimExtensions
    {
        /// <summary>An atom read action wrapper.
        /// </summary>
        /// <param name="readerWriterLockSlim"></param>
        /// <param name="action"></param>
        /// <exception cref="ArgumentNullException"></exception>
        public static void AtomRead(this ReaderWriterLockSlim readerWriterLockSlim, Action action)
        {
            readerWriterLockSlim.MustNotNull("readerWriterLockSlim");
            action.MustNotNull("action");

            readerWriterLockSlim.EnterReadLock();
            try {
                action();
            }
            finally {
                readerWriterLockSlim.ExitReadLock();
            }
        }
        /// <summary>An atom read func wrapper.
        /// </summary>
        /// <param name="readerWriterLockSlim"></param>
        /// <param name="function"></param>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        /// <exception cref="ArgumentNullException"></exception>
        public static T AtomRead<T>(this ReaderWriterLockSlim readerWriterLockSlim, Func<T> function)
        {
            readerWriterLockSlim.MustNotNull("readerWriterLockSlim");
            function.MustNotNull("function");

            readerWriterLockSlim.EnterReadLock();
            try {
                return function();
            }
            finally {
                readerWriterLockSlim.ExitReadLock();
            }
        }
        /// <summary>An atom write action wrapper.
        /// </summary>
        /// <param name="readerWriterLockSlim"></param>
        /// <param name="action"></param>
        /// <exception cref="ArgumentNullException"></exception>
        public static void AtomWrite(this ReaderWriterLockSlim readerWriterLockSlim, Action action)
        {
            readerWriterLockSlim.MustNotNull("readerWriterLockSlim");
            action.MustNotNull("action");

            readerWriterLockSlim.EnterWriteLock();
            try {
                action();
            }
            finally {
                readerWriterLockSlim.ExitWriteLock();
            }
        }
        /// <summary>An atom write func wrapper.
        /// </summary>
        /// <param name="readerWriterLockSlim"></param>
        /// <param name="function"></param>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        /// <exception cref="ArgumentNullException"></exception>
        public static T AtomWrite<T>(this ReaderWriterLockSlim readerWriterLockSlim, Func<T> function)
        {
            readerWriterLockSlim.MustNotNull("readerWriterLockSlim");
            function.MustNotNull("function");

            readerWriterLockSlim.EnterWriteLock();
            try {
                return function();
            }
            finally {
                readerWriterLockSlim.ExitWriteLock();
            }
        }
    }
}
