﻿using System.Text.RegularExpressions;

namespace System
{
    public static class StringExtensions
    {
        /// <summary>
        /// 判定字符串是不是数值型
        /// </summary>
        /// <param name="str">要判断的字符串</param>
        /// <returns></returns>
        public static bool IsNumeric(this string str)
        {
            return Regex.IsMatch(str, @"^[-]?[0-9]*$");
        }

        /// <summary>
        /// 判断字符串是不是yyyy-mm-dd字符串
        /// </summary>
        /// <param name="str">要判断的字符串</param>
        /// <returns>判断结果</returns>
        public static bool IsDate(this string str)
        {
            return Regex.IsMatch(str, @"(\d{4})-(\d{1,2})-(\d{1,2})");
        }

        /// <summary>
        /// 判断字符串是不是时间格式
        /// </summary>
        /// <param name="str">要判断的字符串</param>
        /// <returns></returns>
        public static bool IsTime(this string str)
        {
            return Regex.IsMatch(str, @"^((([0-1]?[0-9])|(2[0-3])):([0-5]?[0-9])(:[0-5]?[0-9])?)$");
        }

        /// <summary>
        /// 判断字符串是不是日期模式
        /// </summary>
        /// <param name="str">要判断的字符串</param>
        /// <returns></returns>
        public static bool IsDateTime(this string str)
        {
            return Regex.IsMatch(str, @"(\d{4})-(\d{1,2})-(\d{1,2}) ^((([0-1]?[0-9])|(2[0-3])):([0-5]?[0-9])(:[0-5]?[0-9])?)$");
        }

        /// <summary>
        /// 判断字符串是不是小数类型
        /// </summary>
        /// <param name="str">要判断的字符串</param>
        /// <returns></returns>
        public static bool IsDecimal(this string str)
        {
            return Regex.IsMatch(str, @"^[-]?[0-9]*[.]?[0-9]*$");
        }

        /// <summary>
        /// 检测是否符合email格式
        /// </summary>
        /// <param name="str">要判断的email字符串</param>
        /// <returns>判断结果</returns>
        public static bool IsEmail(this string str)
        {
            return Regex.IsMatch(str, @"^([\w-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([\w-]+\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)$");
        }

        /// <summary>
        /// 转换为bool型
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <returns>转换后的bool类型结果</returns>
        public static bool ToBoolean(this string str)
        {
            return ToBoolean(str, false);
        }

        /// <summary>
        /// 转换为bool型
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <param name="value">默认值</param>
        /// <returns>转换后的bool类型结果</returns>
        public static bool ToBoolean(this string str, bool value)
        {
            bool val;
            if (bool.TryParse(str, out val)) {
                return val;
            }
            return value;
        }

        /// <summary>
        /// 转换为Int32类型
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <returns></returns>
        public static int ToInt(this string str)
        {
            return ToInt(str, 0);
        }

        /// <summary>
        /// 转换为Int32类型
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <param name="value">缺省值</param>
        /// <returns>转换后的int类型结果</returns>
        public static int ToInt(this string str, int value)
        {
            int val;
            if (int.TryParse(str, out val)) {
                return val;
            }
            return value;
        }


        /// <summary>
        /// 转换为decimal型
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <returns></returns>
        public static decimal ToDecimal(this string str)
        {
            return ToDecimal(str, 0);
        }

        /// <summary>
        /// 转换为decimal型
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <param name="value">缺省值</param>
        /// <returns></returns>
        public static decimal ToDecimal(this string str, decimal value)
        {
            decimal val;
            if (decimal.TryParse(str, out val)) {
                return val;
            }
            return value;
        }


        /// <summary>
        /// 转换为DateTime型
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <returns></returns>
        public static DateTime ToDate(this string str)
        {
            return ToDate(str, DateTime.Today);
        }

        /// <summary>
        /// 转换为DateTime型
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <param name="value">缺省值</param>
        /// <returns></returns>
        public static DateTime ToDate(this string str, DateTime value)
        {
            DateTime date;
            if (DateTime.TryParse(str, out date)) {
                return date;
            }
            return value;
        }
    }
}
