﻿using System;

namespace ThinkNet.Infrastructure
{
    /// <summary>
    /// 实体基类
    /// </summary>
    /// <typeparam name="TIdentify"></typeparam>
    [Serializable]
    public abstract class Entity<TIdentify> : IEntity<TIdentify>
    {
        protected Entity()
        { }

        protected Entity(TIdentify id)
        {
            this.Id = id;
        }


        internal object uniqueId;
        /// <summary>
        /// 标识ID
        /// </summary>
        public virtual TIdentify Id
        {
            get { return Utils.ConvertType<TIdentify>(uniqueId); }
            protected set { uniqueId = value; }
        }

        public override bool Equals(object obj)
        {
            if (obj == null || !(obj is IEntity<TIdentify>)) {
                return false;
            }

            if (Object.ReferenceEquals(this, obj)) {
                return true;
            }

            IEntity<TIdentify> entity = obj as IEntity<TIdentify>;

            return entity.GetType() == this.GetType() && (entity).Id.Equals(this.Id);
        }

        [NonSerialized]
        private int? _requestedHashCode;
        public override int GetHashCode()
        {
            if (!_requestedHashCode.HasValue) {
                _requestedHashCode = this.Id.GetHashCode() ^ 31;
            }

            return _requestedHashCode.Value;
        }

        public static bool operator ==(Entity<TIdentify> left, Entity<TIdentify> right)
        {
            return Object.Equals(left, null)
                ? (Object.Equals(right, null))
                : left.Equals(right);
        }

        public static bool operator !=(Entity<TIdentify> left, Entity<TIdentify> right)
        {
            return !(left == right);
        }


        object IEntity.UniqueId
        {
            get { return this.uniqueId; }
        }
    }
}
