﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;

namespace ThinkNet.Infrastructure
{
    /// <summary>
    /// 程序集初始化接口
    /// </summary>
    public interface IAssemblyInitializer
    {
        /// <summary>
        /// 初始化
        /// </summary>
        /// <param name="container"></param>
        /// <param name="assemblies"></param>
        void Initialize(IObjectContainer container, IEnumerable<Assembly> assemblies);
    }

    public abstract class AssemblyInitializer : IAssemblyInitializer
    {
        public void Initialize(IObjectContainer container, IEnumerable<Assembly> assemblies)
        {
            assemblies
                .SelectMany(assembly => assembly.GetTypes().Where(IsRegisterType))
                .ToList()
                .ForEach(type => RegisterType(type, container));
        }

        protected virtual void RegisterType(Type type, IObjectContainer container)
        {
            var interfaceTypes = type.GetInterfaces().Where(IsGenericType).ToArray();

            if (type.GetAttributes<ParticipateInContainerAttribute>(false).Any()) {
                var attribute = type.GetAttribute<ParticipateInContainerAttribute>(false);
                attribute.RegisterTypes = interfaceTypes;
                attribute.Register(container, type);

                return;
            }

            foreach (var interfaceType in interfaceTypes) {
                if (this.RegisteredTypeName)
                    container.RegisterType(interfaceType, type, type.FullName, LifetimeStyle.Transient);
                else
                    container.RegisterType(interfaceType, type, LifetimeStyle.Transient);
            }
        }

        protected virtual bool RegisteredTypeName
        {
            get { return true; }
        }

        protected abstract bool IsGenericType(Type type);

        protected abstract bool IsRegisterType(Type type);
    }
}
