﻿using System;
using System.Collections.Generic;

namespace ThinkNet.Infrastructure
{
    /// <summary>
    /// 生命周期类型
    /// </summary>
    public enum LifetimeStyle : byte
    {
        /// <summary>
        /// 每次都构造一个实例
        /// </summary>
        Transient,
        /// <summary>
        /// 单例
        /// </summary>
        Singleton,
        /// <summary>
        /// 每个线程
        /// </summary>
        PerThread,
        /// <summary>
        /// 每个请求
        /// </summary>
        PerRequest
    }

    /// <summary>
    /// IOC容器接口
    /// </summary>
    public interface IObjectContainer : IDisposable
    {
        /// <summary>
        /// 判断此类型是否已注册
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        bool IsRegistered(Type type);
        /// <summary>
        /// 判断此类型是否已注册
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        bool IsRegistered(Type type, string name);

        /// <summary>
        /// 向IOC容器注册类型
        /// </summary>
        /// <param name="type">注册类型</param>
        /// <param name="instance">对应的生成实例</param>
        /// <param name="lifetimeStyle">生命周期类型</param>
        IObjectContainer RegisterInstance(Type type, object instance, LifetimeStyle lifetimeStyle);
        /// <summary>
        /// 向IOC容器注册类型
        /// </summary>
        /// <param name="type">注册类型</param>
        /// <param name="instance">对应的生成实例</param>
        /// <param name="name">注册名称</param>
        /// <param name="lifetimeStyle">生命周期类型</param>
        IObjectContainer RegisterInstance(Type type, object instance, string name, LifetimeStyle lifetimeStyle);
        /// <summary>
        /// 向IOC容器注册类型
        /// </summary>
        /// <param name="type">注册类型</param>
        /// <param name="lifetimeStyle">生命周期类型</param>
        IObjectContainer RegisterType(Type type, LifetimeStyle lifetimeStyle);
        /// <summary>
        /// 向IOC容器注册类型
        /// </summary>
        /// <param name="type">注册类型</param>
        /// <param name="name">注册的名称</param>
        /// <param name="lifetimeStyle">生命周期类型</param>
        IObjectContainer RegisterType(Type type, string name, LifetimeStyle lifetimeStyle);
        /// <summary>
        /// 向IOC容器注册类型
        /// </summary>
        /// <param name="from"></param>
        /// <param name="to"></param>
        /// <param name="lifetimeStyle">生命周期类型</param>
        IObjectContainer RegisterType(Type from, Type to, LifetimeStyle lifetimeStyle);
        /// <summary>
        /// 向IOC容器注册类型
        /// </summary>
        /// <param name="from"></param>
        /// <param name="to"></param>
        /// <param name="name">注册的名称</param>
        /// <param name="lifetimeStyle">生命周期类型</param>
        IObjectContainer RegisterType(Type from, Type to, string name, LifetimeStyle lifetimeStyle);

        /// <summary>
        /// 获取类型对应的实例
        /// </summary>
        /// <param name="type">注册类型</param>
        /// <returns></returns>
        object Resolve(Type type);
        /// <summary>
        /// 获取类型对应的实例
        /// </summary>
        /// <param name="type">注册类型</param>
        /// <param name="name">注册的名称</param>
        /// <returns></returns>
        object Resolve(Type type, string name);
        /// <summary>
        /// 获取类型所有的实例
        /// </summary>
        /// <param name="type">注册类型</param>
        /// <returns></returns>
        IEnumerable<object> ResolveAll(Type type);
    }
}
