﻿using System.Configuration;

namespace ThinkNet.Infrastructure.Mail.Configuration
{
    /// <summary>
    /// SMTP认证信息配置
    /// </summary>
    public sealed class SmtpPassportConfigSection : ConfigurationSection
    {
        private static readonly ConfigurationProperty _propSmtpConfigs = new ConfigurationProperty(null, typeof(SmtpPassportSettingsCollection), null, ConfigurationPropertyOptions.IsDefaultCollection);
        private static ConfigurationPropertyCollection _properties = new ConfigurationPropertyCollection();

        static SmtpPassportConfigSection()
        {
            _properties.Add(_propSmtpConfigs);
        }

        /// <summary>
        /// 自定义对象
        /// </summary>
        /// <returns>自定义对象</returns>
        protected override object GetRuntimeObject()
        {
            this.SetReadOnly();
            return this;
        }

        /// <summary>
        /// 默认的配置节点名称
        /// </summary>
        public const string sectionPrefix = "smtpPassports";

        /// <summary>
        /// 获取默认的配置信息
        /// </summary>
        /// <returns></returns>
        public static SmtpPassportConfigSection GetSection()
        {
            object obj = ConfigurationManager.GetSection(sectionPrefix);
            if (obj != null && obj.GetType() == typeof(SmtpPassportConfigSection)) {
                return (SmtpPassportConfigSection)obj;
            }

            return null;
        }


        /// <summary>
        /// 分组节点
        /// </summary>
        [ConfigurationProperty(null, Options = ConfigurationPropertyOptions.IsDefaultCollection)]
        public SmtpPassportSettingsCollection PassportSettings
        {
            get
            {
                return (SmtpPassportSettingsCollection)base[_propSmtpConfigs];
            }
        }

        /// <summary>
        /// 配置信息集合
        /// </summary>
        protected override ConfigurationPropertyCollection Properties
        {
            get
            {
                return _properties;
            }
        }
    }
}
