﻿using System.Configuration;

namespace ThinkNet.Infrastructure.Mail.Configuration
{
    /// <summary>
    /// SMTP 帐号信息
    /// </summary>
    public sealed class SmtpPassportSettings : ConfigurationElement
    {
        private static ConfigurationPropertyCollection _properties = new ConfigurationPropertyCollection();

        /// <summary>
        /// 根据节点名称获取配置信息
        /// </summary>
        /// <param name="key">节点名称</param>
        /// <returns></returns>
        public static SmtpPassportSettings GetSetting(string key)
        {
            SmtpPassportConfigSection groups = SmtpPassportConfigSection.GetSection();

            if (groups != null) {
                foreach (SmtpPassportSettings config in groups.PassportSettings) {
                    if (string.Equals(config.Name.ToLower(), key.ToLower())) {
                        return config;
                    }
                }
            }

            return null;
        }

        static SmtpPassportSettings()
        {
            _properties.Add(_propName);
            _properties.Add(_propHost);
            _properties.Add(_propPort);
            _properties.Add(_propSsl);
            _properties.Add(_propUid);
            _properties.Add(_propPwd);
            //_properties.Add(_propFrom);
            //_properties.Add(_propFromname);
        }

        /// <summary>
        /// 构造新实例
        /// </summary>
        public SmtpPassportSettings()
        { }

        /// <summary>
        /// 构造新实例
        /// </summary>
        /// <param name="name">键名</param>
        /// <param name="host">主机</param>
        /// <param name="port">端口</param>
        public SmtpPassportSettings(string name, string host, int port)
        {
            this.Name = name;
            this.RemoteHost = host;
            this.Port = port;
        }

        /// <summary>
        /// 键
        /// </summary>
        internal string Key
        {
            get
            {
                return this.Name;
            }
        }

        private static readonly ConfigurationProperty _propName = new ConfigurationProperty("name", typeof(string), null, null, new StringValidator(1), ConfigurationPropertyOptions.IsKey | ConfigurationPropertyOptions.IsRequired);
        /// <summary>
        /// 键名
        /// </summary>
        [ConfigurationProperty("name", Options = ConfigurationPropertyOptions.IsKey | ConfigurationPropertyOptions.IsRequired, DefaultValue = "")]
        public string Name
        {
            get
            {
                return (string)base[_propName];
            }
            set
            {
                base[_propName] = value;
            }
        }

        private static readonly ConfigurationProperty _propHost = new ConfigurationProperty("host", typeof(string), null, ConfigurationPropertyOptions.IsRequired);
        /// <summary>
        /// 远程主机(IPaddress or domain)
        /// </summary>
        [ConfigurationProperty("host", Options = ConfigurationPropertyOptions.IsRequired)]
        public string RemoteHost
        {
            get { return (string)base[_propHost]; }
            set { base[_propHost] = value; }
        }

        private static readonly ConfigurationProperty _propUid = new ConfigurationProperty("uid", typeof(string), null, ConfigurationPropertyOptions.IsRequired);
        /// <summary>
        /// 登录名
        /// </summary>
        [ConfigurationProperty("uid", Options = ConfigurationPropertyOptions.IsRequired)]
        public string UserID
        {
            get { return (string)base[_propUid]; }
            set { base[_propUid] = value; }
        }

        private static readonly ConfigurationProperty _propPwd = new ConfigurationProperty("pwd", typeof(string), null, ConfigurationPropertyOptions.IsRequired);
        /// <summary>
        /// 登录密码
        /// </summary>
        [ConfigurationProperty("pwd", Options = ConfigurationPropertyOptions.IsRequired)]
        public string Password
        {
            get { return (string)base[_propPwd]; }
            set { base[_propPwd] = value; }
        }

        private static readonly ConfigurationProperty _propPort = new ConfigurationProperty("port", typeof(int), 21, ConfigurationPropertyOptions.None);
        /// <summary>
        /// 端口
        /// </summary>
        [ConfigurationProperty("port", Options = ConfigurationPropertyOptions.None, DefaultValue = 21)]
        public int Port
        {
            get { return (int)base[_propPort]; }
            set { base[_propPort] = value; }
        }

        private static readonly ConfigurationProperty _propSsl = new ConfigurationProperty("ssl", typeof(bool), false, ConfigurationPropertyOptions.None);
        /// <summary>
        /// 是否安全连接
        /// </summary>
        [ConfigurationProperty("ssl", Options = ConfigurationPropertyOptions.None, DefaultValue = false)]
        public bool SSL
        {
            get { return (bool)base[_propSsl]; }
            set { base[_propSsl] = value; }
        }

        //private static readonly ConfigurationProperty _propFrom = new ConfigurationProperty("from", typeof(string), null, ConfigurationPropertyOptions.IsRequired);
        ///// <summary>
        ///// 发件人地址
        ///// </summary>
        //[ConfigurationProperty("from", Options = ConfigurationPropertyOptions.IsRequired)]
        //public string From
        //{
        //    get { return (string)base[_propFrom]; }
        //    set { base[_propFrom] = value; }
        //}

        //private static readonly ConfigurationProperty _propFromname = new ConfigurationProperty("fromName", typeof(string), "", ConfigurationPropertyOptions.None);
        ///// <summary>
        ///// 发件人姓名
        ///// </summary>
        //[ConfigurationProperty("fromName", Options = ConfigurationPropertyOptions.None, DefaultValue = "")]
        //public string FromName
        //{
        //    get { return (string)base[_propFromname]; }
        //    set { base[_propFromname] = value; }
        //}

        /// <summary>
        /// 配置属性集合
        /// </summary>
        protected override ConfigurationPropertyCollection Properties
        {
            get
            {
                return _properties;
            }
        }
    }
}
