﻿using System;
using System.Configuration;

namespace ThinkNet.Infrastructure.Mail.Configuration
{
    /// <summary>
    /// SMTP账号信息集合
    /// </summary>
    [ConfigurationCollection(typeof(SmtpPassportSettings))]
    public sealed class SmtpPassportSettingsCollection : ConfigurationElementCollection
    {
        private static ConfigurationPropertyCollection _properties = new ConfigurationPropertyCollection();

        /// <summary>
        /// 构造新实例
        /// </summary>
        public SmtpPassportSettingsCollection()
            : base(StringComparer.OrdinalIgnoreCase)
        { }

        /// <summary>
        /// 添加节点配置
        /// </summary>
        /// <param name="settings"></param>
        public void Add(SmtpPassportSettings settings)
        {
            this.BaseAdd(settings);
        }

        /// <summary>
        /// 重写父类方法
        /// </summary>
        /// <param name="index"></param>
        /// <param name="element"></param>
        protected override void BaseAdd(int index, ConfigurationElement element)
        {
            if (index == -1) {
                base.BaseAdd(element, false);
            }
            else {
                base.BaseAdd(index, element);
            }
        }
        /// <summary>
        /// 重写父类方法
        /// </summary>
        /// <returns></returns>
        protected override ConfigurationElement CreateNewElement()
        {
            return new SmtpPassportSettings();
        }

        /// <summary>
        /// 重写父类方法
        /// </summary>
        /// <param name="element"></param>
        /// <returns></returns>
        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((SmtpPassportSettings)element).Key;
        }

        /// <summary>
        /// 查询配置所在集合的索引
        /// </summary>
        /// <param name="settings"></param>
        /// <returns></returns>
        public int IndexOf(SmtpPassportSettings settings)
        {
            return base.BaseIndexOf(settings);
        }

        /// <summary>
        /// 获取指定键的元素
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public new SmtpPassportSettings this[string name]
        {
            get
            {
                return (SmtpPassportSettings)base.BaseGet(name);
            }
        }

        /// <summary>
        /// 获取位置索引的元素
        /// </summary>
        /// <param name="index"></param>
        /// <returns></returns>
        public SmtpPassportSettings this[int index]
        {
            get
            {
                return (SmtpPassportSettings)base.BaseGet(index);
            }
        }

        /// <summary>
        /// 配置集合
        /// </summary>
        protected override ConfigurationPropertyCollection Properties
        {
            get
            {
                return _properties;
            }
        }
    }
}
