﻿using ThinkNet.Infrastructure.Mail.Configuration;

namespace ThinkNet.Infrastructure.Mail
{
    /// <summary>
    /// 邮件认证信息
    /// </summary>
    public abstract class MailAuthentication
    {
        /// <summary>
        /// 初始化 <see cref="MailAuthentication"/> 类的新实例。
        /// </summary>
        protected MailAuthentication()
        { }
        /// <summary>
        /// 使用配置信息来初始化 <see cref="MailAuthentication"/> 类的新实例。
        /// </summary>
        /// <param name="key">配置节点</param>
        protected MailAuthentication(string key)
        {
            SmtpPassportSettings mailconfig = SmtpPassportSettings.GetSetting(key);

            this.MailServer = mailconfig.RemoteHost;
            this.Port = mailconfig.Port;
            this.UserID = mailconfig.UserID;
            this.Password = mailconfig.Password;
            this.EnableSSL = mailconfig.SSL;
        }
        /// <summary>
        /// 使用服务器地址、端口、用户名和密码来初始化 <see cref="MailAuthentication"/> 类的新实例。
        /// </summary>
        /// <param name="server">服务器地址(Ip或域名)</param>
        /// <param name="port">端口</param>
        /// <param name="uid">用户名</param>
        /// <param name="pwd">密码</param>
        protected MailAuthentication(string server, int port, string uid, string pwd)
        {
            this.MailServer = server;
            this.Port = port;
            this.UserID = uid;
            this.Password = pwd;
        }


        /// <summary>
        /// 邮件服务器域名
        /// </summary>	
        public string MailServer { get; set; }

        /// <summary>
        /// 邮件服务器端口号
        /// </summary>	
        public int Port { get; set; }

        /// <summary>
        /// SMTP认证时使用的用户名
        /// </summary>
        public string UserID { get; set; }

        /// <summary>
        /// SMTP认证时使用的密码
        /// </summary>
        public string Password { get; set; }

        /// <summary>
        /// 是否安全连接
        /// </summary>
        public bool EnableSSL { get; set; }
    }
}
