﻿using System;
using System.Collections.Generic;

namespace ThinkNet.Infrastructure
{
    /// <summary>
    /// 分页数据类
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class PagedResult<T>
    {
        /// <summary>
        /// 空数据
        /// </summary>
        public static readonly PagedResult<T> Empty = new PagedResult<T>();

        private PagedResult()
        { }

        /// <summary>
        /// 构造实例
        /// </summary>
        /// <param name="totalRecords">总记录个数</param>
        /// <param name="totalPages">总页数</param>
        /// <param name="pageSize">每页显示记录</param>
        /// <param name="pageIndex">当前页索引</param>
        /// <param name="data">页面数据</param>
        public PagedResult(int totalRecords, int pageSize, int pageIndex, IEnumerable<T> data)
        {
            this.TotalRecords = totalRecords;
            this.PageSize = pageSize;
            this.PageIndex = pageIndex;
            this.TotalPages = (int)Math.Ceiling((double)totalRecords / (double)pageSize);

            this.Data = data;
        }

        /// <summary>
        /// 获取或设置总记录数。
        /// </summary>
        public int TotalRecords { get; private set; }
        /// <summary>
        /// 获取或设置页数。
        /// </summary>
        public int TotalPages { get; private set; }
        /// <summary>
        /// 获取或设置页面大小。
        /// </summary>
        public int PageSize { get; private set; }
        /// <summary>
        /// 获取或设置页码。
        /// </summary>
        public int PageIndex { get; private set; }
        /// <summary>
        /// 获取或设置页面数据
        /// </summary>
        public IEnumerable<T> Data { get; private set; }
    }
}
