﻿using ThinkNet.Domain;

namespace ThinkNet.Infrastructure
{
    /// <summary>
    /// 仓储接口
    /// </summary>
    /// <typeparam name="TAggregateRoot"></typeparam>
    public interface IRepository<TAggregateRoot>
        where TAggregateRoot : class, IAggregateRoot
    {
        /// <summary>
        /// 当前工作单元
        /// </summary>
        IUnitOfWork UnitOfWork { get; }

        /// <summary>
        /// 判断此 <paramref name="aggregateRoot"/> 是否存在
        /// </summary>
        /// <param name="aggregateRoot"></param>
        /// <returns></returns>
        bool Contains(TAggregateRoot aggregateRoot);
        /// <summary>
        /// 保存
        /// </summary>
        /// <param name="aggregateRoot"></param>
        void Save(TAggregateRoot aggregateRoot);
        /// <summary>
        /// 更新
        /// </summary>
        /// <param name="aggregateRoot"></param>
        void Update(TAggregateRoot aggregateRoot);
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="item"></param>
        void Delete(TAggregateRoot aggregateRoot);
        ///// <summary>
        ///// 持久化
        ///// </summary>
        ///// <param name="item"></param>
        //void Persist(TAggregateRoot item);
        /// <summary>
        /// 刷新
        /// </summary>
        /// <param name="aggregateRoot"></param>
        void Refresh(TAggregateRoot aggregateRoot);
        ///// <summary>
        ///// 合并
        ///// </summary>
        ///// <param name="item"></param>
        //void Merge(TAggregateRoot item);
        /// <summary>
        /// 分离
        /// </summary>
        /// <param name="aggregateRoot"></param>
        void Detach(TAggregateRoot aggregateRoot);

        /// <summary>
        /// 根据标识id获得实体
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        TAggregateRoot Load<TIdentify>(TIdentify id);
    }

    ///// <summary>
    ///// 仓储接口
    ///// </summary>
    ///// <typeparam name="TAggregateRoot">聚合根类型</typeparam>
    //public interface IRepository<TAggregateRoot, TIdentify> : IRepository<TAggregateRoot>
    //    where TAggregateRoot : class, IAggregateRoot
    //{
    //    /// <summary>
    //    /// 根据标识id获得实体
    //    /// </summary>
    //    /// <param name="id">标识id</param>
    //    /// <returns></returns>
    //    TAggregateRoot Get(TIdentify id);
    //}
}
