﻿using System;
using System.Data;

using ThinkNet.Domain;


namespace ThinkNet.Infrastructure
{
    /// <summary>
    /// 实现仓储事务的上下文接口
    /// </summary>
    public interface IRepositoryContext : IUnitOfWork, IDisposable
    {
        /// <summary>
        /// 判断此 <paramref name="entity"/> 是否存在于工作单元中
        /// </summary>
        /// <param name="entity"></param>
        /// <returns></returns>
        bool Contains(IEntity entity);

        /// <summary>
        /// 保存(如果标识符由数据库提供，可能会触发sql-insert)
        /// </summary>
        /// <param name="entity">要保存的实体</param>
        void Save(IEntity entity);
        /// <summary>
        /// 更新(提交时才会触发sql-update)
        /// </summary>
        /// <param name="entity">要更新的实体</param>
        void Update(IEntity entity);
        /// <summary>
        /// 删除(提交时才会触发sql-delete)
        /// </summary>
        /// <param name="entity">要删除的实体</param>
        void Delete(IEntity entity);
        ///// <summary>
        ///// 持久化(提交时才会触发sql-insert)
        ///// </summary>
        ///// <param name="entity">要持久化的实体</param>
        //void Persist(IEntity entity);
        /// <summary>
        /// 从数据中刷新(触发sql-select)
        /// </summary>
        /// <param name="entity">要刷新的实体</param>
        void Refresh(IEntity entity);
        ///// <summary>
        ///// 合并到数据库中的实例
        ///// </summary>
        ///// <typeparam name="TAggregateRoot">聚合根类型</typeparam>
        ///// <param name="aggregateRoot">要注册的对象</param>
        //void Merge<TAggregateRoot>(TAggregateRoot aggregateRoot)
        //    where TAggregateRoot : class, IAggregateRoot;
        /// <summary>
        /// 从工作单元中分离此 <paramref name="entity"/>
        /// </summary>
        /// <param name="entity">要分离的实体</param>
        void Detach(IEntity entity);



        //TEntity Get<TEntity, TIdentify>(TIdentify id)
        //    where TEntity : class, IEntity;


        TEntity Load<TEntity>(object id)
            where TEntity : class, IEntity;


        IDbConnection DbConnection { get; }
    }
}
