﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Linq.Expressions;

namespace ThinkNet.Infrastructure
{
    public class Sort<T> : ISort<T>
    {
        public Sort()
        { }

        private Sort<T> Add(ISortItem<T> order)
        {
            orders.Add(order);

            return this;
        }

        /// <summary>
        /// 升序
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <param name="expression"></param>
        public static Sort<T> OrderBy(Expression<Func<T, object>> expression)
        {
            expression.MustNotNull("expression");

            return new Sort<T>().Add(new SortItem(expression, SortOrder.Ascending));
        }
        /// <summary>
        /// 降序
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <param name="expression">排序列</param>
        public static Sort<T> OrderByDescending(Expression<Func<T, object>> expression)
        {
            expression.MustNotNull("expression");

            return new Sort<T>().Add(new SortItem(expression, SortOrder.Descending));
        }


        List<ISortItem<T>> orders = new List<ISortItem<T>>();

        /// <summary>
        /// 升序
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <param name="expression"></param>
        public Sort<T> ThenBy(Expression<Func<T, object>> expression)
        {
            expression.MustNotNull("expression");

            return this.Add(new SortItem(expression, SortOrder.Ascending));
        }
        /// <summary>
        /// 降序
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <param name="expression">排序列</param>
        public Sort<T> ThenByDescending(Expression<Func<T, object>> expression)
        {
            expression.MustNotNull("expression");

            return this.Add(new SortItem(expression, SortOrder.Descending));
        }


        #region ISortSet<T> 成员

        IEnumerable<ISortItem<T>> ISort<T>.Sorters
        {
            get { return orders; }
        }


        IQueryable<T> ISort<T>.Arranged(IQueryable<T> enumerable)
        {
            if (!orders.Any()) {
                return enumerable;
            }

            IOrderedQueryable<T> orderenumerable = null;

            ISortItem<T> first = orders.First();
            switch (first.SortOrder) {
                case SortOrder.Ascending:
                    orderenumerable = enumerable.OrderBy(first.Expression);
                    break;
                case SortOrder.Descending:
                    orderenumerable = enumerable.OrderByDescending(first.Expression);
                    break;
            }


            foreach (ISortItem<T> sort in orders.Skip(1)) {
                switch (sort.SortOrder) {
                    case SortOrder.Ascending:
                        orderenumerable = orderenumerable.ThenBy(sort.Expression);
                        break;
                    case SortOrder.Descending:
                        orderenumerable = orderenumerable.ThenByDescending(sort.Expression);
                        break;
                }
            }

            return orderenumerable;
        }

        #endregion


        internal class SortItem : ISortItem<T>
        {
            ///// <summary>
            ///// 升序
            ///// </summary>
            ///// <typeparam name="TKey"></typeparam>
            ///// <param name="expression"></param>
            //public static ISort<T> ByAscending(Expression<Func<T, object>> expression)
            //{
            //    return new Sort(expression, SortOrder.Ascending);
            //}
            ///// <summary>
            ///// 降序
            ///// </summary>
            ///// <typeparam name="TKey"></typeparam>
            ///// <param name="expression">排序列</param>
            //public static ISort<T> ByDescending(Expression<Func<T, object>> expression)
            //{
            //    return new Sort(expression, SortOrder.Descending);
            //}


            //protected Sort()
            //{ }

            Expression<Func<T, object>> sortPredicate = null;
            SortOrder sortOrder = SortOrder.Unspecified;
            public SortItem(Expression<Func<T, object>> sortPredicate, SortOrder sortOrder)
            {
                //if (sortPredicate == null) {
                //    throw new ArgumentNullException("sortPredicate");
                //}

                //if (sortOrder == SortOrder.Unspecified) {
                //    throw new ArgumentException("必须指定排序方法", "sortOrder");
                //}

                this.sortPredicate = sortPredicate;
                this.sortOrder = sortOrder;
            }

            #region ISort<T> 成员

            Expression<Func<T, object>> ISortItem<T>.Expression
            {
                get { return sortPredicate; }
            }

            SortOrder ISortItem<T>.SortOrder
            {
                get { return sortOrder; }
            }

            #endregion
        }
    }
}
