﻿using System;

namespace ThinkNet.Messaging
{
    public abstract class Message : IMessage
    {
        #region Private Fields
        private readonly Guid id;
        private readonly DateTime timeStamp = DateTime.UtcNow;
        #endregion

        protected Message()
            : this(Guid.NewGuid())
        { }

        protected Message(Guid id)
            : this(id, DateTime.UtcNow)
        { }

        protected Message(Guid id, DateTime timeStamp)
        {
            this.id = id;
            this.timeStamp = timeStamp;
        }

        /// <summary>
        /// 获取消息的全局唯一标识。
        /// </summary>
        public Guid ID
        {
            get { return id; }
        }
        /// <summary>
        /// 获取产生消息的时间戳。
        /// </summary>
        public DateTime TimeStamp
        {
            get { return timeStamp; }
        }


        [NonSerialized]
        private bool _isRestoreFromStorage = false;

        //public bool IsRestoreFromStorage()
        //{
        //    return _isRestoreFromStorage;
        //}

        //public void MarkAsRestoreFromStorage()
        //{
        //    _isRestoreFromStorage = true;
        //}

        bool IMessage.IsRestoreFromStorage()
        {
            return _isRestoreFromStorage;
        }

        void IMessage.MarkAsRestoreFromStorage()
        {
            _isRestoreFromStorage = true;
        }
    }
}
