﻿
namespace ThinkNet.Messaging
{
    /// <summary>The abstract base implementation of IMessageExecutor.
    /// </summary>
    /// <typeparam name="TMessage"></typeparam>
    public abstract class MessageExecutor<TMessage, TMessageQueue> : IMessageExecutor<TMessage, TMessageQueue>
        where TMessage : class, IMessage
        where TMessageQueue : IMessageQueue<TMessage>
    {
        /// <summary>Execute the given message.
        /// </summary>
        /// <param name="message"></param>
        /// <param name="queue"></param>
        public abstract void Execute(TMessage message, TMessageQueue queue);
        /// <summary>Finish the message execution, the message will be removed from the message queue.
        /// </summary>
        /// <param name="message"></param>
        /// <param name="queue"></param>
        protected virtual void FinishExecution(TMessage message, TMessageQueue queue)
        {
            queue.Complete(message);
        }
    }
}
