﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;

namespace ThinkNet
{
    /// <summary>
    /// 常用方法
    /// </summary>
    public sealed class Utils
    {
        #region
        /// <summary>
        /// 返回 HTML 字符串的编码结果
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <returns>编码结果</returns>
        public static string HtmlEncode(string str)
        {
            if (string.IsNullOrEmpty(str))
                return str;

            return HttpUtility.HtmlEncode(str).Replace("'", "&dot");
        }

        /// <summary>
        /// 返回 HTML 字符串的解码结果
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <returns>解码结果</returns>
        public static string HtmlDecode(string str)
        {
            if (string.IsNullOrEmpty(str))
                return str;

            return HttpUtility.HtmlDecode(str).Replace("&dot", "'");
        }

        /// <summary>
        /// 返回 URL 字符串的编码结果
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <returns>编码结果</returns>
        public static string UrlEncode(string str)
        {
            return HttpUtility.UrlEncode(str);
        }

        /// <summary>
        /// 返回 URL 字符串的解码结果
        /// </summary>
        /// <param name="str">要转换的字符串</param>
        /// <returns>解码结果</returns>
        public static string UrlDecode(string str)
        {
            return HttpUtility.UrlDecode(str);
        }

        /// <summary>
        /// 将字符串编码为Base64字符串
        /// </summary>
        /// <param name="str">要编码的字符串</param>
        /// <returns>编码结果</returns>
        public static string Base64Encode(string str)
        {
            return Convert.ToBase64String(Encoding.Default.GetBytes(str));
        }

        /// <summary>
        /// 将Base64字符串解码为普通字符串
        /// </summary>
        /// <param name="str">要解码的字符串</param>
        /// <returns>解码结果</returns>
        public static string Base64Decode(string str)
        {
            return Encoding.Default.GetString(Convert.FromBase64String(str));
        }
        #endregion

        #region
        /// <summary>
        /// 如果前面的值为空，则返回后面的值
        /// </summary>
        /// <param name="strValue">当前值</param>
        /// <param name="defValue">如果为空或null时的值</param>
        /// <returns></returns>
        public static string StrIIF(string strValue, string defValue)
        {
            return string.IsNullOrEmpty(strValue) ? defValue : strValue;
        }

        /// <summary>
        /// 返回字符串真实长度, 1个汉字长度为2
        /// </summary>
        /// <param name="str">字符串</param>
        /// <returns>字符串长度</returns>
        public static int StrTrueLength(string str)
        {
            return Encoding.Default.GetByteCount(str);
        }

        /// <summary>
        /// 取指定长度的字符串，超过部分替代
        /// </summary>
        /// <param name="str">原字符串</param>
        /// <param name="len">指定长度</param>
        /// <param name="strTail">用于替换的字符串</param>
        /// <returns></returns>
        public static string StrCutting(string str, int len, string strTail)
        {
            string result = string.Empty;// 最终返回的结果
            int byteLen = StrTrueLength(str);// 单字节字符长度
            int charLen = str.Length;// 把字符平等对待时的字符串长度
            int byteCount = 0;// 记录读取进度
            int pos = 0;// 记录截取位置
            if (byteLen > len) {
                for (int i = 0; i < charLen; i++) {
                    if (Convert.ToInt32(str.ToCharArray()[i]) > 255) {// 按中文字符计算加2
                        byteCount += 2;
                    }
                    else {// 按英文字符计算加1
                        byteCount += 1;
                    }

                    if (byteCount > len) {// 超出时只记下上一个有效位置
                        pos = i;
                        break;
                    }
                    else if (byteCount == len) {// 记下当前位置
                        pos = i + 1;
                        break;
                    }
                }

                if (pos >= 0) {
                    result = str.Substring(0, pos) + strTail;
                }
            }
            else {
                result = str;
            }

            return result;
        }

        /// <summary>
        /// 分割字符串
        /// </summary>
        /// <param name="strArray">字符串内容</param>
        /// <param name="strSplit">分割字符</param>
        /// <returns>数组</returns>
        public static string[] StrSplit(string strArray, string strSplit)
        {
            if (string.IsNullOrEmpty(strArray))
                return new string[0];

            string[] array = Regex.Split(strArray, Regex.Escape(strSplit), RegexOptions.IgnoreCase);

            List<string> list = new List<string>();
            Array.ForEach<string>(array, delegate(string item) {
                string str = item.Trim();
                if (!string.IsNullOrEmpty(str)) {
                    list.Add(str);
                }
            });
            return list.ToArray();
            //return (from piece in Regex.Split(strArray, Regex.Escape(strSplit), RegexOptions.IgnoreCase)
            //        let trimmed = piece.Trim()
            //        where !string.IsNullOrEmpty(trimmed)
            //        select trimmed).ToArray();
        }

        /// <summary>
        /// 过滤字符串数组中每个元素为合适的大小
        /// 当长度小于minLength时，忽略掉,-1为不限制最小长度
        /// 当长度大于maxLength时，取其前maxLength位
        /// 如果数组中有null元素，会被忽略掉
        /// </summary>
        /// <param name="strArray">源数组</param>
        /// <param name="minLength">单个元素最小长度</param>
        /// <param name="maxLength">单个元素最大长度</param>
        /// <returns></returns>
        public static string[] ArrayIgnore(string[] strArray, int minLength, int maxLength)
        {
            if (minLength > maxLength) {
                int t = maxLength;
                maxLength = minLength;
                minLength = t;
            }

            int iMiniStringCount = 0;
            for (int i = 0; i < strArray.Length; i++) {
                if (minLength > -1 && strArray[i].Length < minLength) {
                    strArray[i] = null;
                    continue;
                }
                if (strArray[i].Length > maxLength) {
                    strArray[i] = strArray[i].Substring(0, maxLength);
                }
                iMiniStringCount++;
            }

            string[] result = new string[iMiniStringCount];
            for (int i = 0, j = 0; i < strArray.Length && j < result.Length; i++) {
                if (strArray[i] != null && strArray[i] != string.Empty) {
                    result[j] = strArray[i];
                    j++;
                }
            }

            return result;
        }

        /// <summary>
        /// 清除字符串数组中的重复项
        /// </summary>
        /// <param name="strArray">字符串数组</param>
        /// <returns></returns>
        public static string[] ArrayDistinct(string[] strArray)
        {
            return ArrayDistinct(strArray, 0);
        }

        /// <summary>
        /// 清除字符串数组中的重复项
        /// </summary>
        /// <param name="strArray">字符串数组</param>
        /// <param name="maxElementLength">字符串数组中单个元素的最大长度</param>
        /// <returns></returns>
        public static string[] ArrayDistinct(string[] strArray, int maxElementLength)
        {
            Hashtable h = new Hashtable();

            foreach (string s in strArray) {
                string k = s;
                if (maxElementLength > 0 && k.Length > maxElementLength) {
                    k = k.Substring(0, maxElementLength);
                }
                h[k.Trim()] = s;
            }

            string[] result = new string[h.Count];

            h.Keys.CopyTo(result, 0);

            return result;
        }

        /// <summary>
        /// 清除数组中的重复项
        /// </summary>
        /// <typeparam name="T">数据类型</typeparam>
        /// <param name="array">一个数组</param>
        /// <returns>不重复的数组</returns>
        public static T[] ArrayDistinct<T>(T[] array)
        {
            Hashtable ht = new Hashtable();
            Array.ForEach<T>(array, delegate(T item) {
                ht[item] = item;
            });
            T[] result = new T[ht.Count];

            ht.Keys.CopyTo(result, 0);

            return result;
        }


        /// <summary>
        /// 判断指定字符串是否属于指定字符串数组中的一个元素
        /// </summary>
        /// <param name="strSearch">要查找的字符串</param>
        /// <param name="stringArray">字符串数组</param>
        /// <param name="caseInsensetive">是否不区分大小写, true为不区分, false为区分</param>
        /// <returns></returns>
        public static bool StrInArray(string strSearch, string[] stringArray, bool caseInsensetive)
        {
            return Array.Exists<string>(stringArray, delegate(string strItem) {
                if (caseInsensetive) {
                    return string.Equals(strSearch, strItem, StringComparison.OrdinalIgnoreCase);
                }
                return string.Equals(strSearch, strItem);
            });
        }

        /// <summary>
        /// 判断指定字符串在指定字符串数组中的位置
        /// </summary>
        /// <param name="strSearch">要查找的字符串</param>
        /// <param name="stringArray">字符串数组</param>
        /// <param name="caseInsensetive">是否不区分大小写, true为不区分, false为区分</param>
        /// <returns></returns>
        public static int StrInArrayIndexof(string strSearch, string[] stringArray, bool caseInsensetive)
        {
            return Array.FindIndex<string>(stringArray, delegate(string strItem) {
                if (caseInsensetive) {
                    return string.Equals(strSearch, strItem, StringComparison.OrdinalIgnoreCase);
                }
                return string.Equals(strSearch, strItem);
            });
        }

        /// <summary>
        /// 判断指定字符串是否属于指定字符串数组中的一个元素
        /// </summary>
        /// <param name="str">字符串</param>
        /// <param name="stringarray">内部以逗号分割单词的字符串</param>
        /// <param name="strsplit">分割字符串</param>
        /// <param name="caseInsensetive">是否不区分大小写, true为不区分, false为区分</param>
        /// <returns>判断结果</returns>
        public static bool StrInArray(string str, string stringarray, string strsplit, bool caseInsensetive)
        {
            return StrInArray(str, StrSplit(stringarray, strsplit), caseInsensetive);
        }

        /// <summary>
        /// 判断指定字符串是否属于指定字符串数组中的一个元素
        /// </summary>
        /// <param name="str">字符串</param>
        /// <param name="stringarray">内部以逗号分割单词的字符串</param>
        /// <param name="strsplit">分割字符串</param>
        /// <returns>判断结果</returns>
        public static bool StrInArray(string str, string stringarray, string strsplit)
        {
            return StrInArray(str, stringarray, strsplit, false);
        }

        /// <summary>
        /// 判断指定字符串是否属于指定字符串数组中的一个元素
        /// </summary>
        /// <param name="str">字符串</param>
        /// <param name="stringarray">内部以逗号(,)分割单词的字符串</param>
        /// <returns>判断结果</returns>
        public static bool StrInArray(string str, string stringarray)
        {
            return StrInArray(str, stringarray, ",");
        }

        /// <summary>
        /// 返回指定IP是否在指定的IP数组所限定的范围内, IP数组内的IP地址可以使用*表示该IP段任意, 例如192.168.1.*
        /// </summary>
        /// <param name="ip">指定的IP地址</param>
        /// <param name="iparray">IP地址范围</param>
        /// <returns>判断结果</returns>
        public static bool StrInIPArray(string ip, string[] iparray)
        {
            string[] userip = StrSplit(ip, @".");
            for (int ipIndex = 0; ipIndex < iparray.Length; ipIndex++) {
                string[] tmpip = StrSplit(iparray[ipIndex], @".");
                int r = 0;
                for (int i = 0; i < tmpip.Length; i++) {
                    if (tmpip[i] == "*") {
                        return true;
                    }

                    if (userip.Length > i) {
                        if (tmpip[i] == userip[i]) {
                            r++;
                        }
                        else {
                            break;
                        }
                    }
                    else {
                        break;
                    }

                }
                if (r == 4) {
                    return true;
                }
            }
            return false;
        }
        #endregion

        #region
        /// <summary>
        /// SHA256函数
        /// </summary>
        /// <param name="str">原始字符串</param>
        /// <returns>SHA256结果</returns>
        public static string SHA256(string str)
        {
            byte[] SHA256Data = Encoding.UTF8.GetBytes(str);
            SHA256Managed Sha256 = new SHA256Managed();
            byte[] Result = Sha256.ComputeHash(SHA256Data);
            return Convert.ToBase64String(Result);  //返回长度为44字节的字符串
        }

        /// <summary>
        /// MD5函数
        /// </summary>
        /// <param name="str">原始字符串</param>
        /// <returns>MD5结果</returns>
        public static string MD5(string str)
        {
            if (string.IsNullOrEmpty(str))
                return str;

            byte[] b = Encoding.Default.GetBytes(str);
            b = new MD5CryptoServiceProvider().ComputeHash(b);
            string ret = "";
            for (int i = 0; i < b.Length; i++) {
                ret += b[i].ToString("x").PadLeft(2, '0');
            }
            return ret;
        }
        #endregion

        #region
        /// <summary>
        /// 格式化字节数字符串
        /// </summary>
        /// <param name="bytes">字节数</param>
        /// <returns></returns>
        public static string FormatBytesStr(int bytes)
        {
            if (bytes > 1073741824) {
                return ((double)(bytes / 1073741824)).ToString("0") + "G";
            }
            if (bytes > 1048576) {
                return ((double)(bytes / 1048576)).ToString("0") + "M";
            }
            if (bytes > 1024) {
                return ((double)(bytes / 1024)).ToString("0") + "K";
            }
            return bytes.ToString() + "Bytes";
        }

        /// <summary>
        /// 计算时间的差别
        /// </summary>
        /// <param name="dt">日期</param>
        /// <returns></returns>
        public static string FormatDateStr(DateTime dt)
        {
            TimeSpan now = new TimeSpan(DateTime.Now.Ticks);
            TimeSpan sou = new TimeSpan(dt.Ticks);
            TimeSpan ts = now.Subtract(sou).Duration();
            if (ts.Days > 0) {
                if (ts.Days > 180) {
                    return dt.ToString("yyyy-MM-dd HH:mm");
                }
                else if (ts.Hours > 90) {
                    return "三个月前";
                }
                else if (ts.Days > 30) {
                    return "一个月前";
                }
                else {
                    return string.Format("{0}天前", ts.Days);
                }
            }
            if (ts.Hours > 0) {
                return string.Format("{0}小时前", ts.Hours);
            }
            if (ts.Minutes > 0) {
                return string.Format("{0}分钟前", ts.Minutes);
            }

            return string.Format("{0}秒前", ts.Seconds);
        }
        #endregion

        #region
        /// <summary>
        /// 获得当前绝对路径
        /// </summary>
        /// <param name="strPath">指定的路径</param>
        /// <returns>绝对路径</returns>
        public static string GetMapPath(string strPath)
        {
            if (HttpContext.Current != null) {
                return HttpContext.Current.Server.MapPath(strPath);
            }
            else //非web程序引用
            {
                return Path.Combine(AppDomain.CurrentDomain.BaseDirectory, strPath);
            }
        }

        /// <summary>
        /// 文件移动
        /// </summary>
        /// <param name="sourceFileName">移动文件</param>
        /// <param name="destFileName">目标文件</param>
        /// <returns>是否成功</returns>
        public static bool FileMove(string sourceFileName, string destFileName)
        {
            if (!File.Exists(sourceFileName))
                return false;

            if (string.IsNullOrEmpty(destFileName))
                return false;

            string destFilePath = destFileName.Substring(0, destFileName.LastIndexOf("\\"));

            if (!Directory.Exists(destFilePath)) {
                Directory.CreateDirectory(destFilePath);
            }
            File.Delete(destFileName);
            File.Move(sourceFileName, destFileName);

            return true;
        }

        /// <summary>
        /// 文件复制
        /// </summary>
        /// <param name="sourceFileName">复制文件</param>
        /// <param name="destFileName">目标文件</param>
        public static bool FileCopy(string sourceFileName, string destFileName)
        {
            if (!File.Exists(sourceFileName))
                return false;

            if (destFileName == string.Empty)
                return false;

            string destFilePath = destFileName.Substring(0, destFileName.LastIndexOf("\\"));

            if (!Directory.Exists(destFilePath)) {
                Directory.CreateDirectory(destFilePath);
            }
            File.Copy(sourceFileName, destFileName, true);

            return true;
        }

        /// <summary>
        /// 判断文件流是否为UTF8字符集
        /// </summary>
        /// <param name="sbInputStream">文件流</param>
        /// <returns>判断结果</returns>
        public static bool StreamIsUTF8(Stream sbInputStream)
        {
            int i;
            byte cOctets;  // octets to go in this UTF-8 encoded character 
            byte chr;
            bool bAllAscii = true;
            long iLen = sbInputStream.Length;

            cOctets = 0;
            for (i = 0; i < iLen; i++) {
                chr = (byte)sbInputStream.ReadByte();

                if ((chr & 0x80) != 0)
                    bAllAscii = false;

                if (cOctets == 0) {
                    if (chr >= 0x80) {
                        do {
                            chr <<= 1;
                            cOctets++;
                        }
                        while ((chr & 0x80) != 0);

                        cOctets--;
                        if (cOctets == 0)
                            return false;
                    }
                }
                else {
                    if ((chr & 0xC0) != 0x80) {
                        return false;
                    }
                    cOctets--;
                }
            }

            if (cOctets > 0) {
                return false;
            }

            if (bAllAscii) {
                return false;
            }

            return true;
        }
        #endregion

        #region
        /// <summary>
        /// 获取当月第一天
        /// </summary>
        public static DateTime MonthFirstDay
        {
            get
            {
                return new DateTime(DateTime.Today.Year, DateTime.Today.Month, 1);
            }
        }

        /// <summary>
        /// 获取当月最后一天
        /// </summary>
        public static DateTime MonthLastDay
        {
            get
            {
                return EndDayOfMonth(DateTime.Today.Year, DateTime.Today.Month);
            }
        }

        /// <summary>
        /// 根据年月获取最后一天
        /// </summary>
        /// <param name="year"></param>
        /// <param name="monty"></param>
        /// <returns></returns>
        public static DateTime EndDayOfMonth(int year, int monty)
        {
            return new DateTime(year, monty, DateTime.DaysInMonth(year, monty));
        }
        #endregion

        public static T ConvertType<T>(object value)
        {
            if (value == null) {
                return default(T);
            }

            var typeConverter1 = TypeDescriptor.GetConverter(typeof(T));
            if (typeConverter1.CanConvertFrom(value.GetType())) {
                return (T)typeConverter1.ConvertFrom(value);
            }

            var typeConverter2 = TypeDescriptor.GetConverter(value.GetType());
            if (typeConverter2.CanConvertTo(typeof(T))) {
                return (T)typeConverter2.ConvertTo(value, typeof(T));
            }

            return (T)Convert.ChangeType(value, typeof(T));
        }

        /// <summary>
        /// Gets the hash code for an object based on the given array of hash
        /// codes from each property of the object.
        /// </summary>
        /// <param name="hashCodesForProperties">The array of the hash codes
        /// that are from each property of the object.</param>
        /// <returns>The hash code.</returns>
        public static int GetHashCode(params int[] hashCodesForProperties)
        {
            unchecked {
                int hash = 23;
                foreach (var code in hashCodesForProperties)
                    hash = hash * 29 + code;
                return hash;
            }
        }

        /// <summary>
        /// 数字随机数
        /// </summary>
        /// <param name="len">生成长度</param>
        /// <returns>数字随机数</returns>
        public static string RandomNum(int len)
        {
            StringBuilder num = new StringBuilder();

            Func<int> GetRandomSeed = new Func<int>(() => {
                byte[] bytes = new byte[4];
                RNGCryptoServiceProvider rng = new RNGCryptoServiceProvider();
                rng.GetBytes(bytes);
                return BitConverter.ToInt32(bytes, 0);
            });


            for (int i = 0; i < len; i++) {
                int seek = GetRandomSeed(); //unchecked((int)DateTime.Now.Ticks);

                double random = new Random(seek).NextDouble();
                int number = new Random(seek).Next(2, 16);

                num.Append(random.ToString().Substring(number, 1));
            }

            return num.ToString();
        }

        public static string RandomStr(int length)
        {
            int maxSize = length;
            char[] chars = new char[62];
            string a;
            a = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
            chars = a.ToCharArray();
            int size = maxSize;
            byte[] data = new byte[1];
            RNGCryptoServiceProvider crypto = new RNGCryptoServiceProvider();
            crypto.GetNonZeroBytes(data);
            size = maxSize;
            data = new byte[size];
            crypto.GetNonZeroBytes(data);
            StringBuilder result = new StringBuilder(size);
            foreach (byte b in data) {
                result.Append(chars[b % (chars.Length - 1)]);
            }
            // Unique identifiers cannot begin with 0-9
            //if (result[0] >= '0' && result[0] <= '9') {
            //    return RandomStr(length);
            //}
            return result.ToString();
        }

        /// <summary>
        /// AES加密字符串
        /// </summary>
        /// <param name="encryptString">待加密的字符串</param>
        /// <param name="encryptKey">加密密钥,要求为8位</param>
        /// <returns>加密成功返回加密后的字符串,失败返回源串</returns>
        public static string EncryptWithAES(string encryptString, string encryptKey)
        {
            return AES.Encrypt(encryptString, encryptKey);
        }

        /// <summary>
        /// AES解密字符串
        /// </summary>
        /// <param name="decryptString">待解密的字符串</param>
        /// <param name="decryptKey">解密密钥,要求为8位,和加密密钥相同</param>
        /// <returns>解密成功返回解密后的字符串,失败返源串</returns>
        public static string DecryptWithAES(string decryptString, string decryptKey)
        {
            return AES.Decrypt(decryptString, decryptKey);
        }

        /// <summary>
        /// AES加密字符串
        /// </summary>
        /// <param name="encryptString">待加密的字符串</param>
        /// <param name="encryptKey">加密密钥,要求为8位</param>
        /// <returns>加密成功返回加密后的字符串,失败返回源串</returns>
        public static string EncryptWithDES(string encryptString, string encryptKey)
        {
            return DES.Encrypt(encryptString, encryptKey);
        }

        /// <summary>
        /// AES解密字符串
        /// </summary>
        /// <param name="decryptString">待解密的字符串</param>
        /// <param name="decryptKey">解密密钥,要求为8位,和加密密钥相同</param>
        /// <returns>解密成功返回解密后的字符串,失败返源串</returns>
        public static string DecryptWithDES(string decryptString, string decryptKey)
        {
            return DES.Decrypt(decryptString, decryptKey);
        }

        public static string Guid
        {
            get { return System.Guid.NewGuid().ToString().Replace("-", "").ToLower(); }
        }

    }


    internal class AES
    {
        private static byte[] aesKeys = { 0x41, 0x72, 0x65, 0x79, 0x6F, 0x75, 0x6D, 0x79, 0x53, 0x6E, 0x6F, 0x77, 0x6D, 0x61, 0x6E, 0x3F };
        /// <summary>
        /// AES加密字符串
        /// </summary>
        /// <param name="encryptString">待加密的字符串</param>
        /// <param name="encryptKey">加密密钥,要求为8位</param>
        /// <returns>加密成功返回加密后的字符串,失败返回源串</returns>
        public static string Encrypt(string encryptString, string encryptKey)
        {
            //encryptKey = Utils.GetSubString(encryptKey, 32, "");
            encryptKey = encryptKey.Substring(0, 32);
            encryptKey = encryptKey.PadRight(32, ' ');

            RijndaelManaged rijndaelProvider = new RijndaelManaged();
            rijndaelProvider.Key = Encoding.UTF8.GetBytes(encryptKey.Substring(0, 32));
            rijndaelProvider.IV = aesKeys;
            ICryptoTransform rijndaelEncrypt = rijndaelProvider.CreateEncryptor();

            byte[] inputData = Encoding.UTF8.GetBytes(encryptString);
            byte[] encryptedData = rijndaelEncrypt.TransformFinalBlock(inputData, 0, inputData.Length);

            return Convert.ToBase64String(encryptedData);
        }

        /// <summary>
        /// AES解密字符串
        /// </summary>
        /// <param name="decryptString">待解密的字符串</param>
        /// <param name="decryptKey">解密密钥,要求为8位,和加密密钥相同</param>
        /// <returns>解密成功返回解密后的字符串,失败返源串</returns>
        public static string Decrypt(string decryptString, string decryptKey)
        {
            try {
                //decryptKey = Utils.GetSubString(decryptKey, 32, "");
                decryptKey = decryptKey.Substring(0, 32);
                decryptKey = decryptKey.PadRight(32, ' ');

                RijndaelManaged rijndaelProvider = new RijndaelManaged();
                rijndaelProvider.Key = Encoding.UTF8.GetBytes(decryptKey);
                rijndaelProvider.IV = aesKeys;
                ICryptoTransform rijndaelDecrypt = rijndaelProvider.CreateDecryptor();

                byte[] inputData = Convert.FromBase64String(decryptString);
                byte[] decryptedData = rijndaelDecrypt.TransformFinalBlock(inputData, 0, inputData.Length);

                return Encoding.UTF8.GetString(decryptedData);
            }
            catch {
                return string.Empty;
            }
        }
    }

    internal class DES
    {
        private static byte[] desKeys = { 0x12, 0x34, 0x56, 0x78, 0x90, 0xAB, 0xCD, 0xEF };

        /// <summary>
        /// DES加密字符串
        /// </summary>
        /// <param name="encryptString">待加密的字符串</param>
        /// <param name="encryptKey">加密密钥,要求为8位</param>
        /// <returns>加密成功返回加密后的字符串,失败返回源串</returns>
        public static string Encrypt(string encryptString, string encryptKey)
        {
            //encryptKey = Utils.GetSubString(encryptKey, 8, "");
            encryptKey = encryptKey.Substring(0, 8);
            encryptKey = encryptKey.PadRight(8, ' ');
            byte[] rgbKey = Encoding.UTF8.GetBytes(encryptKey.Substring(0, 8));
            byte[] rgbIV = desKeys;
            byte[] inputByteArray = Encoding.UTF8.GetBytes(encryptString);
            DESCryptoServiceProvider dCSP = new DESCryptoServiceProvider();
            MemoryStream mStream = new MemoryStream();
            CryptoStream cStream = new CryptoStream(mStream, dCSP.CreateEncryptor(rgbKey, rgbIV), CryptoStreamMode.Write);
            cStream.Write(inputByteArray, 0, inputByteArray.Length);
            cStream.FlushFinalBlock();
            return Convert.ToBase64String(mStream.ToArray());

        }

        /// <summary>
        /// DES解密字符串
        /// </summary>
        /// <param name="decryptString">待解密的字符串</param>
        /// <param name="decryptKey">解密密钥,要求为8位,和加密密钥相同</param>
        /// <returns>解密成功返回解密后的字符串,失败返源串</returns>
        public static string Decrypt(string decryptString, string decryptKey)
        {
            try {
                //decryptKey = Utils.GetSubString(decryptKey, 8, "");
                decryptKey = decryptKey.Substring(0, 8);
                decryptKey = decryptKey.PadRight(8, ' ');
                byte[] rgbKey = Encoding.UTF8.GetBytes(decryptKey);
                byte[] rgbIV = desKeys;
                byte[] inputByteArray = Convert.FromBase64String(decryptString);
                DESCryptoServiceProvider DCSP = new DESCryptoServiceProvider();

                MemoryStream mStream = new MemoryStream();
                CryptoStream cStream = new CryptoStream(mStream, DCSP.CreateDecryptor(rgbKey, rgbIV), CryptoStreamMode.Write);
                cStream.Write(inputByteArray, 0, inputByteArray.Length);
                cStream.FlushFinalBlock();
                return Encoding.UTF8.GetString(mStream.ToArray());
            }
            catch {
                return string.Empty;
            }

        }
    }
}
